import {avatar, BaseEntity, column, comp, DataSourceStore, datasourcestore, EntityID, t, Table} from "@intermesh/goui";
import {img, JmapDataSource, jmapds} from "@intermesh/groupoffice-core";

interface Employee extends BaseEntity {
	displayName: string,
	avatarId?: string,
	id: EntityID,
	username: string

}

export class EmployeeTable extends Table<DataSourceStore<JmapDataSource<Employee>>> {
	constructor() {
		const store = datasourcestore<JmapDataSource<Employee>, Employee>({
			dataSource: jmapds("Principal"),
			queryParams: {
				limit: 20
			},
			sort: [{property: "name", isAscending: true}],
			filters: {
				isEmployee: {
					isEmployee: true
				}
			}
		});

		const columns = [
			column({
				header: "",
				id: "avatarId",
				resizable: false,
				sortable: false,
				width: 60,
				renderer: (value, record) => {
					return comp({cls: "meta"},
						record?.avatarId ?
							img({
								cls: "goui-avatar",
								blobId: value,
								title: record.name
							}) :

							avatar({
								displayName: record.name
							}));
				}
			}),
			column({
				header: t("Name"),
				id: "name",
				resizable: true,
				width: 312,
				sortable: true,
				renderer: (value, record) => {
					return "<div>" + value + "</div><div><small>" + record.description + "</small></div>";
				}
			})
		];

		super(store, columns);
		this.headers = false;
		this.fitParent = true;

	}
	}
