<?php

namespace go\modules\business\calendartimetracking;

use go\core;
use go\core\orm\Mapping;
use go\core\orm\Query;
use go\modules\business\registration\model\Registration;
use go\modules\community\calendar\model\CalendarEvent;

class Module extends core\Module
{
	public function getAuthor(): string
	{
		return "Intermesh BV <info@intermesh.nl>";
	}

	public function defineListeners()
	{
		parent::defineListeners();

		Registration::on(Registration::EVENT_BEFORE_DELETE, static::class, 'onRegistrationBeforeDelete');

		CalendarEvent::on(CalendarEvent::EVENT_SAVE, static::class, "onCalendarEventSave");
		CalendarEvent::on(CalendarEvent::EVENT_MAPPING, static::class, "onCalendarEventMapping");
	}

	public static function onCalendarEventMapping(Mapping $mapping)
	{
		$mapping->addTable("calendar_time_tracking_time_entry", "te", ["eventId" => "calendarEventId"], ["registrationId"]);
	}

	public static function onCalendarEventSave(CalendarEvent $event): void
	{
		$registration = Registration::findById($event->registrationId);

		if (!empty($registration)) {
			$registration->startDate = $event->start;
			$registration->startTime = $event->start->format('H:i:s');
			$registration->comment = $event->title;

			$start = $event->start->setTimezone($event->timeZone());
			$end = $event->end();
			$interval = $start->diff($end);

			$bookedMinutes = $interval->days * 24 * 60;
			$bookedMinutes += $interval->h * 60;
			$bookedMinutes += $interval->i;

			$registration->bookedMinutes = $bookedMinutes;

			$registration->save();
		}
	}

	public static function onRegistrationBeforeDelete(Query $query)
	{
		$calendarEvent = CalendarEvent::find()
			->select("cce.eventId, cal.aclId, '0'")
			->where("te.registrationId", "in", $query);

		if (!empty($calendarEvent)) {
			CalendarEvent::entityType()->changes($calendarEvent);
		}
	}

	public function requiredLicense(): ?string
	{
		return "groupoffice-pro";
	}

	public function getStatus(): string
	{
		return self::STATUS_BETA;
	}


}