import {CalendarEvent, CalendarItem, CalendarView} from "@intermesh/community/calendar";
import {btn, Component, Menu, t, translate} from "@intermesh/goui";
import {TimeEntryDialog} from "./TimeEntryDialog.js";
import {client, jmapds} from "@intermesh/groupoffice-core";

translate.load(GO.lang.business.calendartimetracking, "business", "calendartimetracking");

const origEventHtml = CalendarView.prototype["eventHtml"];

CalendarView.prototype["eventHtml"] = function (item: CalendarItem, div?: HTMLElement) {
	// @ts-ignore
	if (item.data.registrationId !== undefined && item.data.registrationId !== null) {
		if (!item["extraIcons"].includes("event_available")) {
			item["extraIcons"].push("event_available");
		}
	}

	return origEventHtml.call(this, item, div);
}

type ExtendedCalendarEvent = CalendarItem & { data: (CalendarEvent & { registrationId: string }) };

const origRender = CalendarView.prototype.render;


CalendarView.prototype.render = function (parentEl, insertBefore) {
	this.contextMenu.items.add(
		btn({
			itemId: "cttAddBtn",
			icon: "add",
			text: t("Create time entry", "business", "calendartimetracking"),
			handler: (btn) => {
				const dlg = new TimeEntryDialog();
				dlg.loadCalItem(this["current"]!);
				dlg.show();
			}
		}),
		btn({
			itemId: "cttRemoveBtn",
			icon: "delete",
			text: t("Remove time entry", "business", "calendartimetracking"),
			handler: async () => {
				await jmapds("Registration").confirmDestroy([(this["current"] as ExtendedCalendarEvent).data.registrationId]);

				jmapds("CalendarEvent").updateFromServer();
			},
			disabled: true
		})
	)

	this.contextMenu.on("beforeshow", ({target}) => {
		const addBtn = target.findChild("cttAddBtn");
		const removeBtn = target.findChild("cttRemoveBtn");

		if ((this["current"] as ExtendedCalendarEvent).data.registrationId) {
			addBtn!.disabled = true;
			removeBtn!.disabled = false;
		} else {
			addBtn!.disabled = false;
			removeBtn!.disabled = true;
		}

		jmapds("EmployeeAgreement").query({
			filter: {employeeId: client.user.id}
		}).then((r) => {
			if (r.ids.length === 0) {
				addBtn!.disabled = true;
				removeBtn!.disabled = true;
			}
		})
	})

	return origRender.call(this, parentEl, insertBefore);
}