<?php
namespace go\modules\business\projects3;

use Faker\Generator;
use go\core;
use go\core\model;
use go\core\model\User;
use go\core\orm\Mapping;
use go\modules\business\business\model\Business;
use go\modules\business\business\model\Employee;
use go\modules\business\projects3\model\Project3;
use go\modules\business\projects3\model\ProjectBook;
use go\modules\business\projects3\model\ProjectMilestone;
use go\modules\business\projects3\model\ProjectResource;
use go\modules\community\tasks\model\TaskList;

/**
 * @copyright (c) 2020, Intermesh BV https://www.intermesh.nl
 * @author System Administrat <admin@intermesh.localhost>
 * @license http://www.gnu.org/licenses/agpl-3.0.html AGPLv3
 */
class Module extends core\Module
{
	public function getAuthor() : string
	{
		return "Intermesh BV";
	}

	public function getDependencies() : array
	{
		return ['business/business', 'community/addressbook', "community/tasks"];
	}

	public function requiredLicense(): ?string
	{
		return "groupoffice-pro";
	}

	public function getStatus(): string
	{
		return self::STATUS_BETA;
	}

	public function autoInstall(): bool
	{
		return true;
	}

	protected function afterInstall(model\Module  $model): bool
	{
		foreach(Business::find() as $business) {
			$book = new ProjectBook();
			$book->businessId = $business->id;
			$book->name = $business->name;
			$book->setAcl([model\Group::ID_INTERNAL => model\Acl::LEVEL_WRITE]);
			if(!$book->save()) {
				throw new core\orm\exception\SaveException($book);
			}
		}

		return true;
	}

	public function demo(Generator $faker)
	{
		$business = Business::find()->single();

		$users = User::find(['id', 'displayName', 'email', 'employee'])
			->limit(3)
			->all();

		foreach($users as $user) {
			if(!$user->employee) {
				$employee = new Employee($user);
				$employee->businessId = $business->id;
				$employee->hourlyCosts = 50;
				$employee->hourlyRevenue = 100;
				$user->employee = $employee;

				if(!$user->save()) {
					throw new core\orm\exception\SaveException($user);
				}
			}
		}

		foreach(ProjectBook::find() as $book) {

			$tasklist = TaskList::findById($book->tasklistId);

			for($i = 0; $i < 10; $i++) {
				$project = new Project3();
				$project->bookId = $book->id;
				$project->name = $faker->text(50);
				$project->customerId = \go\modules\community\addressbook\Module::get()->demoCompany($faker)->id;
				$project->contactId = \go\modules\community\addressbook\Module::get()->demoContact($faker)->id;
				$project->startsAt = $faker->dateTimeBetween("-1 years", "now");

				$project->resources = [];
				foreach($users as $user) {
					$project->resources[$user->id] = new ProjectResource($project);
					$project->resources[$user->id] ->budgetMinutes = $faker->numberBetween(2, 200) * 60;
				}

				for($n = 1; $n <= 3; $n++) {
					$milestone = new ProjectMilestone($project);
					$milestone->name = "Phase " . $i;
					$project->milestones[] = $milestone;
				}

				if(!$project->save()) {
					throw new core\orm\exception\SaveException($project);
				}

				if(\go\core\model\Module::isInstalled("community", "tasks")) {
					foreach ($project->milestones as $milestone) {
						\go\modules\community\tasks\Module::get()->demoTasks($faker, $tasklist, false, null, 5, $project->id, $milestone->id);
					}
				}

				if(\go\core\model\Module::isInstalled("legacy", "savemailas")) {
					\GO\Savemailas\SavemailasModule::get()->demoMail($faker, $project);
				}

				echo ".";

			}

		}
	}

}
