create table business_projects3_book
(
    id           int unsigned auto_increment
        primary key,
    aclId        int                                  not null,
    businessId   int unsigned                         not null,
    numberFormat varchar(50)      default 'P%Y-00000' not null,
    name         varchar(190)                         not null,
    tasklistId int unsigned null,
    constraint business_projects3_book_tasklist
     foreign key (tasklistId) references tasks_tasklist (id)
            on delete set null,
    constraint business_projects3_book_pk
        unique (name),
    constraint business_projects3_book_acl
        foreign key (aclId) references core_acl (id)
            on delete cascade,
    constraint business_projects3_book_business
        foreign key (businessId) references business_business (id)
            on delete cascade
);



create index aclId
    on business_projects3_book (aclId);

create index businessId
    on business_projects3_book (businessId);

create table business_projects3_book_status
(
    id      int unsigned auto_increment
        primary key,
    bookId  int unsigned not null,
    name    varchar(190) not null,
    color   char(6)      not null,
    `order` int          not null,
    constraint business_projects3_book_status_business_projects3_book_id_fk
        foreign key (bookId) references business_projects3_book (id)
            on delete cascade
);

create index business_projects3_book_status_order_index
    on business_projects3_book_status (`order`);

create table business_projects3_project3
(
    id             int unsigned auto_increment
        primary key,
    bookId         int unsigned         not null,
    number         varchar(190)         null,
    name           varchar(190)         not null,
    customerId     int                  null,
    contactId      int                 null,
    orderId        int unsigned         null,
    filesFolderId  int(11) unsigned     null,
    startsAt       date             null,
    dueAt          date             null,
    completedAt    date             null,
    timeIsBillable tinyint(1) default 0 null,
    createdBy      int(11) unsigned     not null,
    createdAt      datetime             not null,
    modifiedBy     int(11) unsigned     not null,
    modifiedAt     datetime             null,
    statusId       int unsigned         null,
        constraint business_projects3_book_status_id_fk
        foreign key (statusId) references business_projects3_book_status (id)
        on delete set null,
    constraint business_projects3_project3_ibfk_1
        foreign key (bookId) references business_projects3_book (id)
            on delete cascade,
    constraint business_projects3_project3_ibfk_2
        foreign key (contactId) references addressbook_contact (id)
            on delete set null,

    constraint business_projects3_project3_ibfk_3
        foreign key (customerId) references addressbook_contact (id)
            on delete set null

);

create index contactId
    on business_projects3_project3 (contactId);

create index organizationId
    on business_projects3_project3 (customerId);

create table business_projects3_project3_custom_fields
(
    id int unsigned
        primary key,
    constraint business_projects3_project3_custom_fields_ibfk_1
        foreign key (id) references business_projects3_project3 (id)
            on delete cascade
);

create table business_projects3_project3_resource
(
    employeeId int             not null,
    projectId  int unsigned not null,
    fee        decimal(19, 4)  null,
    budgetMinutes int unsigned null,
    primary key (projectId, employeeId),
    constraint business_projects3_project3_resource_business_employee_id_fk
        foreign key (employeeId) references business_employee (id),
    constraint business_projects3_resource_project_id_fk
        foreign key (projectId) references business_projects3_project3 (id)
            on delete cascade
);


create table business_projects3_book_status_user
(
    id  int unsigned not null,
    userId    int not null,
    modSeq  int          null,
    isVisible boolean      null,
    constraint business_projects3_book_status_user_pk
        primary key (id, userId),
    constraint business_projects3_book_status_user_status_id_fk
        foreign key (id) references business_projects3_book_status (id)
            on delete cascade,
    constraint business_projects3_book_status_user_core_user_id_fk
        foreign key (userId) references core_user (id)
            on delete cascade
);


create table business_projects3_book_user
(
    id      int unsigned not null,
    userId  int          not null,
    modSeq  int          null,
    filter longtext collate utf8mb4_bin null comment '@dbType=json',
    isVisible boolean      null,
    constraint business_projects3_book_user_pk
        primary key (userId, id),
    constraint business_projects3_book_user_business_projects3_book_id_fk
        foreign key (id) references business_projects3_book (id)
            on delete cascade,
    constraint business_projects3_book_user_core_user_i_fk
        foreign key (userId) references core_user (id)
            on delete cascade
);



create table business_projects3_project3_milestone
(
    id int unsigned auto_increment primary key,
    projectId  int unsigned not null,
    name        varchar(100)  null,
    startsAt       date             null,
    dueAt          date             null,
    sortOrder int unsigned,
    constraint business_projects3_milestone_project_id_fk
        foreign key (projectId) references business_projects3_project3 (id)
            on delete cascade
);


alter table tasks_task
    add constraint tasks_task_business_projects3_project3_id_fk
        foreign key (projectId) references business_projects3_project3 (id)
            on delete set null;

alter table tasks_task
    add constraint tasks_task_business_projects3_project3_milestone_id_fk
        foreign key (mileStoneId) references business_projects3_project3_milestone (id)
            on delete set null;