import {client, modules, router} from "@intermesh/groupoffice-core";
import {MainPanel} from "./MainPanel.js";
import {t} from "@intermesh/goui";
import {ProjectWindow} from "./ProjectWindow.js";
import {ProjectDetail} from "./ProjectDetail.js";
import "./Projects2Override.js"

export * from "./ProjectFinance.js";
export * from "./ProjectWindow.js"

modules.register(  {
	package: "business",
	name: "projects3",
	entities: [
		"ProjectBook",
		{
			name:"Project3",
			filters: [
				{
					name: 'text',
					type: "string",
					multiple: false,
					title: t("Query")
				},
				{
					name: 'name',
					type: "string",
					multiple: true,
					title: t("Name")
				},
				{
					name: 'content',
					type: "string",
					multiple: true,
					title: t("Content")
				},
				{
					title: t("Has links to..."),
					name: 'link',
					multiple: false,
					type: 'go.links.FilterLinkEntityCombo'
				},
				{
					title: t("Commented at"),
					name: 'commentedat',
					multiple: false,
					type: 'date'
				}, {
					title: t("Modified at"),
					name: 'modifiedat',
					multiple: false,
					type: 'date'
				}, {
					title: t("Modified by"),
					name: 'modifiedBy',
					multiple: true,
					type: 'string'
				}, {
					title: t("Created at"),
					name: 'createdat',
					multiple: false,
					type: 'date'
				}, {
					title: t("Created by"),
					name: 'createdby',
					multiple: true,
					type: 'string'
				},{
					title: t("Customer"),
					name: 'customerId',
					multiple: true,
					type: 'go.modules.community.addressbook.ContactCombo',
					typeConfig: {
						isOrganization: true
					},
				}
			],

			links: [{
				iconCls: 'entity Project green',
				/**
				 * Opens a dialog to create a new linked item
				 *
				 * @param {string} entity eg. "Note"
				 * @param {string|int} entityId
				 * @returns ProjectWindow
				 */
				linkWindow: (entity:string, entityId) => {
					return new ProjectWindow();
				},

				/**
				 * Return component for the detail view
				 */
				linkDetail () {
					return new ProjectDetail();
				}
			}]
		}
	],
	async init () {
		client.on("authenticated",  ({session}) => {

			modules.getAll();

			if(!session.capabilities["go:business:projects3"]) {
				// User has no access to this module
				return;
			}

			// translate.load(GO.lang.core.core, "core", "core");
			// translate.load(GO.lang.community.tasks, "business", "supportclient");
			// translate.load(GO.lang.business.supportclient, "business", "supportclient");

			const mainPanel = new MainPanel();

			router.add(/^project3\/(\d+)$/, (projectId) => {
				modules.openMainPanel("projects3");
				mainPanel.setProjectId(projectId);
			});

			router.add(/^projects3$/, () => {
				modules.openMainPanel("projects3");
				// mainPanel.setTaskId();
			});

			modules.addMainPanel( "business", "projects3", "projects3", t("Projects"), () => {
				return mainPanel;
			});
		});
	}
});


