import {
	btn, ComboBox,
	combobox,
	comp, Component, ContainerFieldValue,
	datefield,
	DateInterval,
	DateTime,
	DisplayField,
	displayfield,
	fieldset, Form,
	form,
	Format,
	NumberField,
	numberfield,
	t,
	tbar,
	Window
} from "@intermesh/goui";

import {RegistrationTable} from "@intermesh/business/registration";
import {client, jmapds, router} from "@intermesh/groupoffice-core";

export class InvoiceWindow extends Window {
	private invoiceAmountField!: NumberField;
	private invoicePercentageField!: NumberField;
	private registrationTable: RegistrationTable;
	private billHoursAmountField: DisplayField;
	private billHoursField: DisplayField;
	private form: Form<ContainerFieldValue>;
	private quoteInfo: Component;
	constructor(projectData:any) {
		super();

		this.width  = 800;
		this.height = 1000;
		this.resizable = true;

		this.title = t("Create invoice");


		this.items.add(
			this.form = form({flex: 1, cls: "vbox", handler: form1 => {
						const data = form1.value;

						return client.jmap("Project3/createInvoice", {
							projectId: projectData.projectId,
							bookId: data.bookId,
							percentage: data.invoicePercentage,
							quoteId: data.quoteId,
							registrationIds: this.registrationTable.store.data.map(r => r.id)
						}).then(invoice => {
							router.goto("FinanceDocument/" + invoice.id);
							this.close();
						}).catch(reason => {
							void Window.error(reason);
						})

					}},

				fieldset({},

					combobox({
						name: "bookId",
						label: t("Book"),
						dataSource: jmapds("FinanceBook"),
						storeConfig: {
							filters: {
								default: {
									type: "salesinvoice",
									businessId: projectData.businessId
								}
							}
						},
						listeners: {
							render: async ({target}) => {
								const r = await target.list.store.load();
								if(r.length)
									target.value = r[0].id;
							}
						}
					}),

					combobox({
						dataSource: jmapds("FinanceDocument"),
						label: t("Quote"),
						name: "quoteId",
						filterName: "text",
						displayProperty: "number",
						disabled: projectData.quoteIds.length == 0,
						value: projectData.quoteIds.length ? projectData.quoteIds[0] : undefined,
						storeConfig: {
							sort: [{property: "number", isAscending: true}],
							filters: {
								default: {
									id: projectData.quoteIds
								}
							}
						},
						listeners: {
							setvalue: async ( {newValue}) => {

								if(!newValue) {
									this.quoteInfo.hidden = true;
									this.invoiceAmountField.disabled = true;
									this.invoicePercentageField.disabled = true;
									return;
								}
								const quote = await jmapds("FinanceDocument").single(newValue);

								if(!quote) {
									return;
								}

								const outstanding = Math.max(0, projectData.quoteAmount + projectData.billedFee - projectData.invoicedAmount);

								this.invoiceAmountField.max = outstanding;

								this.quoteInfo.hidden = false;
								this.invoiceAmountField.disabled = false;
								this.invoicePercentageField.disabled = false;

								this.form.patch({
									total: quote.subtotalPrice,
									invoicedAmount: projectData.invoicedAmount - projectData.billedFee,
									outstanding: outstanding,
									invoiceAmount: outstanding,
									invoicePercentage: outstanding * 100 / quote.subtotalPrice ,
								})

							}
						}
					}),

					datefield({
						name: "until",
						label: t("Bill hours until"),
						value: (new DateTime()).format("Y-m-d"),
						listeners: {
							change: ( {newValue}) => {
								void this.registrationTable.store
									.setFilter("until", {startDate: "< " +(new DateTime(newValue)).addDays(1).format("Y-m-d")})
									.load();
							}
						}
					}),


				),

				comp({ cls: "hbox"},

					fieldset({legend: t("Billable fee"),  cls: "vbox", flex: 1},

						comp({cls: "vbox label-align-left", flex: 1},
							displayfield({
								label: t("Total"),
								cls: "right",
								value: projectData.billableFee,
								renderer: (v) => Format.money(v)
							}),

							displayfield({
								label: t("Invoiced"),
								cls: "right",
								value: projectData.billedFee,
								renderer: (v) => Format.money(-v)
							}),

							displayfield({
								cls: "right border-top",
								label: t("Outstanding"),
								value: projectData.billableFee - projectData.billedFee,
								renderer: (v) => Format.money(v)
							}),

							// displayfield({
							// 	cls: "right",
							// 	label: t("Billed (h)"),
							// 	value: projectData.billedMinutes,
							//
							// 	renderer: (v) => {
							// 		const d = DateInterval.createFromFormat(v, "j")!;
							// 		return d.format("h:I");
							// 	}
							// }),
						),
					),

					fieldset({legend: t("Quote"),  cls: "vbox", flex: 1},



						this.quoteInfo = comp({cls:"label-align-left vbox", hidden: true},

							displayfield({
								name: "total",
								cls: "right",
								label: t("Total"),
								renderer: (v) => Format.money(v)
							}),

							displayfield({
								name: "invoicedAmount",
								cls: "right",
								label: t("Invoiced"),
								renderer: (v) => Format.money(v)
							}),

							displayfield({
								name: "outstanding",
								cls: "right border-top",
								label: t("Outstanding"),
								renderer: (v) => Format.money(v)
							})
						)
					),



				),

				fieldset({legend: "New invoice"},

					comp({cls: "hbox gap"},


						comp({flex: 1, cls: "flow"},


							comp({flex: 1, cls: "vbox label-align-left"},
								this.billHoursField = displayfield({
									cls: "right",
									label: t("Hours"),
									value: 0,
									renderer:(v) => {
										const d = DateInterval.createFromFormat(v, "j")!;
										return d.format("h:I");
									}
								}),

								this.billHoursAmountField = displayfield({
									cls: "right",
									label: t("Amount"),
									value: 0,
									renderer: (v) => Format.money(v)
								}),
							)
						),

						comp({cls: "vbox", flex: 1},

							comp({flex: 1}),

							comp({cls: "group"},

								this.invoiceAmountField = numberfield({
									flex: 1,
									disabled: true,
									name: "invoiceAmount",
									label: t("Amount"),
									decimals: 2,
									value: 0,
									min: 0,
									max: 0,
									listeners: {
										change: ({target, newValue}) => {
											if(target.isValid()) {
												const total = this.form.findField("total")!.value as number;
												this.invoicePercentageField.value = newValue * 100 / total;
											}
										}
									}
								}),
								this.invoicePercentageField = numberfield({
									flex: 1,
									disabled: true,
									name: "invoicePercentage",
									label: t("Percentage"),
									decimals: 0,
									min: 0,
									max: 100,
									value: 0 ,
									listeners: {
										change: ({target, newValue}) => {
											if(target.isValid()) {
												const total = this.form.findField("total")!.value as number;
												this.invoiceAmountField.value = projectData.quoteAmount * newValue / 100;
											}
										}
									}
								})
							)
						),

					)
				),

				comp({flex: 1, cls: "scroll bg-lowest"},
				this.registrationTable = new RegistrationTable()
				),

				tbar({},
					"->",
					btn({
						text: t("Create invoice"),
						type: "submit"
					})
				)
			)
		);

		this.registrationTable.scrollLoad = false;
		this.registrationTable.store.queryParams.limit = 0;

		this.registrationTable.store.on("datachanged", ({target}) => {
			let totalMinutes = 0, totalBillableFee=0;
			target.forEach((rec) => {
				totalMinutes += rec.bookedMinutes
				if(rec.hourlyRevenue) {
					totalBillableFee += rec.hourlyRevenue / 60 * rec.bookedMinutes;
				}
			})

			this.billHoursAmountField.value  = totalBillableFee;
			this.billHoursField.value = totalMinutes;
		})


		void this.registrationTable.store
			.setFilter( "defaults",{
				projectId: projectData.projectId,
				billed: false,
				billable: true
			})
			.setFilter("until", {startDate: "< " +(new DateTime()).addDays(1).format("Y-m-d")})
			.load();

	}
}