import {
	btn,
	column,
	Component,
	DateInterval,
	EntityID,
	h3,
	Store,
	store,
	StoreRecord,
	t,
	Table,
	table,
	tbar
} from "@intermesh/goui";
import {client, jmapds} from "@intermesh/groupoffice-core";

export class ProjectActivityTime extends Component {
	private table: Table<Store<StoreRecord>>;
	constructor() {
		super();
		this.cls = "card";

		this.hidden = true;

		this.table = this.createTable();

		this.items.add(
			tbar({}, h3("Time per activity"), "->", btn({
				icon: "expand_less",
				handler: (btn) => {
					this.table.hidden = !this.table.hidden;

					btn.icon = this.table.hidden ? "expand_more" : "expand_less";
				}
			})),
			this.table
		);
	}


	private createTable() {
		return table({
			// cls: "fit",
			fitParent: true,
			columns:[
				column({
					resizable: true,
					id: "activity",
					header: t("Activity"),
					renderer: (columnValue, record, td, table, storeIndex) => {

						return columnValue.name;

					}
				}),

				column({
					resizable: true,
					align: "right",
					id: "bookedMinutes",
					width: 100,
					header: t("Booked"),
					renderer:(columnValue, record, td, table1) => {
						if(!columnValue) {
							return "0:00";
						}
						const di = DateInterval.createFromFormat(columnValue, "j")!;

						return di.format("h:I");
					}
				})
			],
			store: store()
		})
	}


	public async load(projectId: EntityID) {

		const response = await client.jmap("Registration/activities", {projectId: projectId});

		if(!response.activities.length) {
			this.hide();
			return;
		} else {
			this.show();
		}

		await Promise.all(response.activities.map(async (r:any) => {
			r.activity = await jmapds("Activity").single(r.activityId);
		}))

		this.table.store.loadData(response.activities, false);
	}
}