import {
	btn,
	comp,
	DataSourceForm,
	datasourceform,
	displaydatefield,
	displayfield, fieldset,
	Format,
	hr,
	menu,
	t,
	tbar
} from "@intermesh/goui";
import {addbutton, DetailPanel, filesbutton, jmapds, linkbrowserbutton, modules} from "@intermesh/groupoffice-core";
import {ProjectWindow} from "./ProjectWindow.js";
import {StatusButton} from "./StatusButton.js";
import {Project} from "./Project.js";
import {ProjectResourceDetail} from "./ProjectResourceDetail.js";
import {ProjectActivityTime} from "./ProjectActivityTime.js";
import {ProjectTasksDetail} from "./ProjectTasksDetail.js";
import {ProjectFinance} from "./ProjectFinance";


export class ProjectDetail extends DetailPanel<Project> {
	private form: DataSourceForm<Project>;
	private statusButton: StatusButton;
	private resources: ProjectResourceDetail;
	private activities: ProjectActivityTime;
	private tasks: ProjectTasksDetail;
	private finance: ProjectFinance;

	constructor() {
		super("Project3");

		this.scroller.items.add(
			this.form = datasourceform({
				dataSource: jmapds("Project3")
			},

				comp({cls: "card "},

					tbar({},
					this.titleCmp = comp({tagName: "h3", flex: 1}),
						this.statusButton = new StatusButton()
					),

					comp({cls: "flow"},

						comp({flex: 1},

							displaydatefield({
								name: "startsAt",
								label:t("Starts at")
							}),

							displaydatefield({
								name: "dueAt",
								label:t("Due at")
							}),

							displaydatefield({
								name: "completedAt",
								label:t("Completed at")
							}),
						),

						comp({
								flex: 1
							},

							displayfield({

								name: "customerId",
								label:t("Customer"),
								icon: "business",
								escapeValue: false,
								renderer: async (v) => {
									if(!v) {
										return "";
									}
									const c = await jmapds("Contact").single(v);
									return c ? `<a href="#contact/${c.id}">${c.name.htmlEncode()}</a>` : "";
								}
							}),

							displayfield({

								name: "contactId",
								label:t("Contact"),
								icon: "person",
								escapeValue: false,
								renderer: async (v) => {
									if(!v) {
										return "";
									}
									const c = await jmapds("Contact").single(v);
									return c ? `<a href="#contact/${c.id}">${c.name.htmlEncode()}</a>` : "";
								}
							})
						)
					)
				)
			),

			this.finance = new ProjectFinance(),

			this.resources = new ProjectResourceDetail(),

			this.activities = new ProjectActivityTime(),

			this.tasks = new ProjectTasksDetail()


		)

		this.addCustomFields();
		this.addComments();
		this.addFiles();
		this.addLinks();
		this.addHistory();

		this.toolbar.items.add(

			btn({
				icon: "edit",
				title: t("Edit"),
				handler: (button, ev) => {
					const dlg = new ProjectWindow();
					void dlg.load(this.entity!.id);
					dlg.show();
				}
			}),

			addbutton(),

			linkbrowserbutton(),

			btn({
				icon: "more_vert",
				menu: menu({},
					btn({
						icon: "print",
						text: t("Print"),
						handler: () => {
							this.print();
						}
					}),

					hr(),

					btn({
						icon: "delete",
						text: t("Delete"),
						handler: () => {
							jmapds("Project3").confirmDestroy([this.entity!.id]);
						}
					})
					)
			})
		)

		if(modules.isAvailable("legacy", "files")) {
			this.toolbar.items.insert(-1, filesbutton())
		}

		this.on("load",( {entity}) => {
			this.title = entity.number + ": " + entity.name;
			void this.form.load(entity.id);
			void this.statusButton.setProject(entity);

			if(modules.isAvailable("business", "registration")) {
				void this.resources.load(entity.id);
				void this.activities.load(entity.id);
			}

			if(modules.isAvailable("business", "finance")) {
				void this.finance.load(entity.id);
			}
			if(modules.isAvailable("community", "tasks")) {
				void this.tasks.load(entity.id);
			}
		});

		this.on("reset", () => {
			this.form.reset();
		});
	}

}