import {
	btn,
	comp,
	Component,
	ContainerFieldValue,
	DateTime,
	displayfield,
	EntityID,
	Fieldset,
	fieldset,
	Form,
	form,
	Format,
	h3,
	t,
	tbar
} from "@intermesh/goui";
import {Chart, client, jmapds, modules, router} from "@intermesh/groupoffice-core";
import {InvoiceWindow} from "./InvoiceWindow.js";

export class ProjectFinance extends Component {
	private form: Form<ContainerFieldValue>;
	private projectId?: EntityID;
	private hoursChart?: Chart<any, number[], string>;
	private profitChart?: Chart<any, number[], string>;
	private billableChart?: Chart<any, number[], string>;
	constructor() {
		super();

		this.cls = "card project-finance";
		this.hidden = true;

		this.itemId = Component.uniqueID();

		jmapds("Registration").on("change", this.onChange, {bind: this});
		jmapds("FinanceDocument").on("change", this.onChange, {bind: this});
		this.on("remove", () => {
			jmapds("Registration").un("change", this.onChange);
			jmapds("FinanceDocument").un("change", this.onChange);
		})

		this.items.add(
			this.form = form({
					cls: "label-align-left",
					listeners: {
						beforesetvalue: (e) => {


							e.value.tasksTotal = e.value.tasksTodo + e.value.tasksCompleted;
							e.value.billableTotal = e.value.quoteAmount + e.value.billableFee;

							e.value.profit = e.value.invoicedAmount - e.value.purchasedAmount - e.value.bookedFeeCosts - e.value.billableFeeCosts;
							e.value.totalMinutes = e.value.bookedMinutes + e.value.billableMinutes;
							e.value.outstandingMinutes = e.value.billableMinutes - e.value.billedMinutes;
							e.value.totalFeeCosts = e.value.billableFeeCosts + e.value.bookedFeeCosts;
							e.value.outstandingBillableFee = e.value.billableFee - e.value.billedFee;

							if( e.value.quoteAmount) {
								e.value.outstandingForQuote = e.value.quoteAmount + e.value.billedFee - e.value.invoicedAmount ;
								e.value.invoicedForQuote = e.value.invoicedAmount - e.value.billedFee;
							} else {
								e.value.outstandingForQuote = 0
								e.value.invoicedForQuote = 0
							}

							if(e.value.billableTotal === 0) {
								this.hide();
							} else {
								this.show();
							}

							this.drawHoursChart(e.value);
							this.drawBillableChart(e.value);
							this.drawProfitChart(e.value);

						}
					}
				},
				tbar({}, h3(t("Finance"))),

				comp({cls: "project-finance-graphs"},

					fieldset({legend: "Hours", cls: "vbox gap"},

						comp({tagName: "canvas", id: "project-finance-hours-chart-" + this.itemId}),

						displayfield({
							cls: "right",
							name: "budgetMinutes",
							label: t("Estimated hours"),
							renderer: (v) => {
								return Format.duration(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "bookedMinutes",
							label: t("Actual hours"),
							renderer: (v) => {
								return Format.duration(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "billableMinutes",
							label: t("Additional hours"),
							renderer: (v, field) => {
								field.previousSibling()!.el.classList.toggle("border-bottom", v > 0);
								return Format.duration(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "billedMinutes",
							label: t("Billed"),
							renderer: (v) => {
								return Format.duration(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "outstandingMinutes",
							label: t("Outstanding billable hours"),
							renderer: (v) => {
								 return Format.duration(v);
							}
						}),

						comp({flex: 1}), // fill space till bottom in hbox layout

						...(modules.isAvailable("business", "registration") ? [
							comp({flex: 1}), // fill space till bottom in hbox layout
							btn({
								cls: "outlined",
								text: t("View time entries"),
								handler: () => {
									let url = "registration/list/" + this.projectId + "//billable/<" + (new DateTime()).addDays(1).format("Y-m-d");
									void router.goto(url);
								}

							})] : [])
					),

					fieldset({legend: "Invoiced", cls: "vbox gap"},

						comp({tagName: "canvas", id: "project-finance-billable-chart-" + this.itemId}),

						displayfield({
							cls: "right",
							name: "billableFee",
							label: t("Billable fee"),
							renderer: (v, field) => {
								//field.el.classList.toggle("border-bottom", !this.previousSibling()!.hidden);
								return v ? Format.money(v) : "";
							}
						}),

						displayfield({
							cls: "right border-bottom",
							name: "billedFee",
							label: t("Billed fee"),
							renderer: (v, field) => {
								// field.el.classList.toggle("border-bottom", !this.previousSibling()!.hidden);
								return v ? Format.money(v) : "";
							}
						}),

						displayfield({
							cls: "right",
							name: "outstandingBillableFee",
							label: t("Outstanding billable fee"),
							renderer: (v, field) => {

								if(field.previousSibling()!.hidden) {
									return "";
								}
								return Format.money(v);
							}
						}),


						displayfield({
							cls: "right",
							name: "quoteAmount",
							label: t("Quoted amount"),
							renderer: (v) => {

								return v ? Format.money(v) : "";
							}
						}),

						// displayfield({
						// 	cls: "right",
						// 	name: "billableTotal",
						// 	label: t("Total"),
						// 	renderer: (v, field) => {
						// 		//don't show if there's no quoted amount
						// 		// if(field.previousSibling()!.previousSibling()!.hidden){
						// 		// 	return "";
						// 		// }
						// 		return Format.money(v);
						// 	}
						// }),

						displayfield({
							cls: "right",
							name: "invoicedForQuote",
							label: t("Invoiced"),
							renderer: (v, field) => {
								if(field.previousSibling()!.hidden) {
									return "";
								}
								return Format.money(v);
							}
						}),



						displayfield({
							cls: "right",
							name: "outstandingForQuote",
							label: t("Outstanding"),
							renderer: (v, field) => {

								field.previousSibling()!.el.classList.toggle("border-bottom", v >= 0);

								return v >= 0 ? Format.money(v) : "";
							}
						}),

						comp({flex: 1}), // fill space till bottom in hbox layout

						...(modules.isAvailable("business", "registration") ? [btn({
							text: t("Create invoice"),
							cls: "outlined",
							handler: async () => {

								const fv = this.form.value;
								const win = new InvoiceWindow(fv);
								win.show();
							}
						})] : [])
					),

					fieldset({legend: "Profit", cls: "vbox gap"},

						comp({tagName: "canvas", id: "project-finance-profit-chart-" + this.itemId}),

						displayfield({
							cls: "right",
							name: "invoicedAmount",
							label: t("Invoiced"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),


						displayfield({
							cls: "right",
							name: "purchasedAmount",
							label: t("Purchases"),
							renderer: (v) => {
								return v ? Format.money(-v) : "";
							}
						}),


						displayfield({
							cls: "right border-bottom",
							name: "totalFeeCosts",
							label: t("Fee cost"),
							renderer: (v) => {
								return Format.money(-v);
							}
						}),

						displayfield({
							cls: "right",
							name: "profit",
							label: t("Profit"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),


					),
				)
			)
		)
	}

	private drawHoursChart(newValue:any) {

		const actualDiff = newValue.budgetMinutes - newValue.bookedMinutes,
			totalHours = newValue.budgetMinutes + newValue.billableMinutes,
			percentage = totalHours?  Math.ceil((newValue.bookedMinutes + newValue.billableMinutes) * 100 / totalHours) + "%" : "-"

		const data = {
			labels: [
				'Budget',
				'Actual hours',
				'Additional billable hours'
			],
			centerLabel: percentage,
			datasets: [{
				data: [
					actualDiff / 60,
					newValue.bookedMinutes / 60,
					newValue.billableMinutes / 60,
				],
				backgroundColor: [
					actualDiff > 0 ? 'rgb(54, 162, 235)' : 'rgb(161,0,40)',
					'rgb(21,172,36)',
					'rgb(255, 99, 132)',
					'rgb(193,155,0)'

				],
				hoverOffset: 4
			}]
		};

		if(!this.hoursChart) {
			this.hoursChart = new Chart("project-finance-hours-chart-" + this.itemId, {
				type: 'doughnut',
				data: data,
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false
						},
						//doughnutCenterLabel: {}
					}
				}
			});
		} else {
			this.hoursChart.data = data;
			this.hoursChart.update();
		}
	}

	private drawBillableChart(newValue:any) {

		let percentage:string = "-", labels, dataSets;
		if(!newValue.quoteAmount) {
			// if there's no quote then billing progress is purely calculated off the billable hours
			percentage = newValue.billableMinutes ? Math.ceil(newValue.billedMinutes * 100 / newValue.billableMinutes) + "%" : '-';

			labels = [
				'Billed fee',
				'Outstanding fee',
			];

			dataSets = [
				newValue.billedFee,
				0,
				newValue.billableFee - newValue.billedFee
			];

		} else {
			const billableAmount = (newValue.quoteAmount + newValue.billableFee)
			percentage = billableAmount ? Math.ceil((100 - ((newValue.outstandingForQuote + newValue.outstandingBillableFee) / billableAmount) * 100)) + "%" : "-";

			labels = [
				'Invoiced of quote',
				'Billed fee',
				'Invoicable for quote',
				'Billable fee',
			];

			dataSets = [
				newValue.invoicedAmount - newValue.billedFee,
				newValue.billedFee,
				newValue.quoteAmount + newValue.billedFee - newValue.invoicedAmount,
				newValue.billableFee - newValue.billedFee
			];
		}

		const data = {
			labels: labels,
			centerLabel: percentage,
			datasets: [{
				data: dataSets,
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(54, 162, 235)',
					'rgb(21,172,36)',
					'rgb(21,172,36)',
				],
				hoverOffset: 4
			}]
		};

		if(!this.billableChart) {
			this.billableChart = new Chart("project-finance-billable-chart-" + this.itemId, {
				type: 'doughnut',
				data: data,
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false
						},
						//doughnutCenterLabel: {}
					}
				}
			});
		} else {
			this.billableChart.data = data;
			this.billableChart.update();
		}
	}

	private drawProfitChart(newValue:any) {

		const margin = newValue.invoicedAmount ?
				Math.ceil((1 - (newValue.purchasedAmount + newValue.bookedFeeCosts + newValue.billableFeeCosts) / newValue.invoicedAmount) * 100) + "%" : "-";

		const data = {
			labels: [
				t('Profit'),
				t('Purchases'),
				t('Fee costs'),
			],
			centerLabel: margin,
			datasets: [{
				data: [
					newValue.invoicedAmount - newValue.purchasedAmount - newValue.bookedFeeCosts- newValue.billableFeeCosts,
					newValue.purchasedAmount,
					newValue.bookedFeeCosts + newValue.billableFeeCosts
				],
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(21,172,36)',
					'rgb(255, 99, 132)',
					'rgb(255, 99, 132)',

				],
				hoverOffset: 4
			}]
		};

		if(!this.profitChart) {
			this.profitChart = new Chart("project-finance-profit-chart-" + this.itemId, {
				type: 'doughnut',
				data: data,
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false
						},
						//doughnutCenterLabel: {}
					}
				}
			});
		} else {
			this.profitChart.data = data;
			this.profitChart.update();
		}
	}

	private onChange() {
		if(this.projectId) {
			this.load(this.projectId);
		}
	}


	public async load(projectId:EntityID) {
		this.projectId = projectId;
		this.form.value  = await client.jmap("Project3/getFinanceData", {projectId: projectId});
	}
}