import {
	btn,
	Component,
	containerfield,
	displayfield,
	durationfield,
	mapfield,
	MapField,
	numberfield,
	t
} from "@intermesh/goui";
import {jmapds, principalcombo} from "@intermesh/groupoffice-core";

export class ProjectResources extends Component {
	private field: MapField;

	constructor() {
		super();
		this.items.add(

			this.field = mapfield({
				keyFieldName: "employeeId",
				cls: "goui-pit",
				name: "resources",
				buildField: (value) => {
					return containerfield({cls: "hbox gap", style: {padding: ".8rem"}},
						displayfield({
							flex: 1,
							// label: t("User"),
							name: "employeeId",
							renderer: (v, field) => {

								if(!v) {
									return "-";
								}
								return jmapds("Principal").single(v).then(user => user ? user.name : "-")
							}
						}),

						numberfield({
							label: t("Fee"),
							placeholder: t("Use default"),
							name: "fee",
							width: 140,
							decimals: 2
						}),

						durationfield({
							label: t("Budget"),
							name: "budgetMinutes",
							outputFormat: "j",
							width: 140
						}),

						btn({
							icon: "delete",
							handler: button => {
								button.parent!.remove();
							}
						})
					)
				}
			}),

			principalcombo({
				entity: "User",
				label: undefined,
				placeholder: t("Add employee"),
				tableConfig: {
					emptyStateHtml: `<div class="goui-empty-state"><i class="icon">person</i><p>${t("No employees yet. Please set a business in the Employee tab of a user via <a class='primary' href='#systemsettings/users'>System settings -> Users</a>.")}</p></div>`
				},
				name: undefined,
				filter: {
					isEmployee: true
				},
				listeners: {
					select: ({target}) => {
						this.field.add({employeeId: target.value}, target.value as string);
						setTimeout(() =>{
							target.reset();
						})
					}
				}
			})
		)

	}
}