import {
	column,
	Config,
	createComponent,
	datasourcestore,
	DataSourceStore,
	datecolumn,
	ListEventMap,
	t,
	Table
} from "@intermesh/goui";
import {jmapds} from "@intermesh/groupoffice-core";
import {StatusButton} from "./StatusButton.js";

export class ProjectTable extends Table<DataSourceStore> {
	constructor() {

		const store = datasourcestore({
			dataSource: jmapds("Project3"),
			relations: {
				customer: {
					dataSource: jmapds("Contact"),
					path: "customerId"
				}
			},
			queryParams: {
				limit: 20
			},
			sort: [{property: "number", isAscending: false}]
		});

		const columns = [
			column({
				header: t("Number"),
				id:"number",
				resizable: true,
				sortable: true,
				width: 120
			}),
			column({
				header: t("Name"),
				id:"name",
				resizable: true,
				sortable: true,
				width: 180
			}),
			column({
				header: t("Status"),
				id:"statusId",
				resizable: true,
				sortable: true,
				width: 180,
				renderer: (columnValue, record, td, table, storeIndex, column1) => {
					const btn =  new StatusButton()
					btn.setProject(record);
					return btn;
				}
			}),
			column({
				header: t("Customer"),
				id:"customer",
				property: "customer/name",
				resizable: true,
				sortable: true,
				width: 200
			}),
			datecolumn({
				header: t("Starts at"),
				id: "startsAt",
				sortable: true
			}),
			datecolumn({
				header: t("Due at"),
				id: "dueAt",
				sortable: true
			}),
			datecolumn({
				header: t("Completed at"),
				id: "completedAt",
				sortable: true
			})
		];
		
		super(store, columns );

		this.rowSelectionConfig =  {
			multiSelect: true
		};

		this.headers = true;

	}
}

export const projecttable = (config?: Config<ProjectTable>) => createComponent(new ProjectTable(), config);