import {btn, checkbox, ComboBox, combobox, comp, datefield, fieldset, t, tbar, textfield} from "@intermesh/goui";
import {FormWindow, jmapds, router} from "@intermesh/groupoffice-core";
import {ProjectResources} from "./ProjectResources.js";
import {ProjectMilestones} from "./ProjectMilestones.js";

export class ProjectWindow extends FormWindow {
	private projectMilestones: ProjectMilestones;

	constructor() {
		super("Project3");

		this.title = t("Project");
		this.stateId = "project-dialog";
		this.maximizable = true;
		this.resizable = true;
		this.width = 640;
		this.hasLinks = true;

		this.on("addlink", async ({entityName, entityId}) => {
			if(entityName == "FinanceDocument") {
				const doc = await jmapds("FinanceDocument").single(entityId);

				if(doc) {
					this.form.patch({
						contactId: doc.contactId,
						customerId: doc.customerId,
						name: doc.reference
					})
				}
			}
		})

		this.form.on("save", ({data, isNew}) => {
			//show detail when creating new project
			if(isNew) {
				router.goto("project3/" + data.id);
			}
		})

		this.generalTab.items.add(
			fieldset({},

				textfield({
					flex: 2,
					name: "name",
					label: t("Name"),
					required: true
				}),

				combobox({
					width: 200,
					dataSource: jmapds("ProjectBook"),
					label: t("Book"),
					name: "bookId",
					filterName: "text",
					required: true,
					selectFirst: true
				}),

				combobox({
					dataSource: jmapds("Contact"),
					label: t("Contact"),
					name: "contactId",
					filterName: "text",
					buttons: [btn({
						icon: "clear",
						handler: (button, ev) => {
							button.findAncestorByType(ComboBox)!.value = null;
						}
					})],
					storeConfig: {
						sort: [{property: "name", isAscending: true}],
						filters: {
							default: {
								isOrganization: false
							}
						}
					},
					listeners: {
						change: async ( {target, newValue}) => {
							const contact = await target.dataSource.single(newValue);

							if(contact && contact.organizationIds.length) {
								const customerField = target.nextSibling() as ComboBox;
								customerField.value = contact.organizationIds[0];
							}

						}
					}
				}),

				combobox({
					dataSource: jmapds("Contact"),
					label: t("Customer"),
					name: "customerId",
					filterName: "text",
					buttons: [btn({
						icon: "clear",
						handler: (button, ev) => {
							button.findAncestorByType(ComboBox)!.value = null;
						}
					})],
					storeConfig: {
						filters: {
							default: {
								isOrganization: true
							}
						}
					}
				}),
			),

			fieldset({},
				datefield({
					label: t("Starts at"),
					name: "startsAt"
				}),

				datefield({
					label: t("Due at"),
					name: "dueAt"
				}),

				datefield({
					label: t("Completed at"),
					name: "completedAt"
				}),

				checkbox({
					name: "timeIsBillable",
					label: t("Time entries are billable for post calculation by default")
				}),
			),

			fieldset({legend: t("Resources")},
				new ProjectResources()
			),

			fieldset({legend: t("Milestones")},
				this.projectMilestones = new ProjectMilestones(),

				tbar({},

					'->',
					btn({
						icon: "add",
						cls: "outlined",
						text: "Add milestone",
						handler: () => {

							this.projectMilestones.addValue({
							});
						}
					})
				),
			)
		);

		this.addCustomFields();

	}

	async setCustomerFromContact () {
		const contactField = this.form.findField("contactId") as ComboBox;
		if(!contactField.value) {
			return;
		}
		const contact = await contactField.dataSource.single(contactField.value + "");

		if(contact && contact.organizationIds.length) {
		const customerField = contactField.nextSibling() as ComboBox;
		customerField.value = contact.organizationIds[0];
	}
}

}