import {btn, Button, DateTime, menu, t} from "@intermesh/goui";
import {Project} from "./Project.js";
import {jmapds} from "@intermesh/groupoffice-core";
import {statuses} from "./Statuses.js";

interface Status {
	id: string
	color:string
	name: string,
	readonly?: boolean
}

export class StatusButton extends Button {
	private project?: Project;
	private statuses: Record<string, Status> = {};

	constructor() {
		super();
		this.menu = menu({renderTo: undefined});
		this.cls = "project3-status";
	}

	public async setProject(project:Project) {

		this.statuses = structuredClone(statuses);

		this.project = project;

		const book = (await jmapds("ProjectBook").single(project.bookId.toString()))!;

		book.statuses.forEach((s:any) => {
			this.statuses[s.id] = s;
		})

		this.buildStatusMenu();

		const statusId = this.getStatusId(project);

		const statusBtn = this.menu!.items.find((value, index, obj) => {
			return value.dataSet.id == statusId;
		});

		if(statusBtn instanceof Button) {
			this.applyOptionBtnValueToMainBtn(statusBtn);
		} else {
			this.text = t("Upcoming");
			this.style.backgroundColor = "#" + statuses['upcoming'].color;
		}
	}

	private getStatusId(project:Project) {
		if(project.completedAt) {
			return "complete";
		}

		if(project.statusId) {
			return project.statusId;
		}

		const now = new DateTime();


		if(project.startsAt && DateTime.createFromFormat(project.startsAt, "Y-m-d")!.compare(now) == -1) {
			return "in-progress";
		} else {
			return "upcoming";
		}
	}


	private menuHandler = (btn:Button) => {

		this.applyOptionBtnValueToMainBtn(btn);

		const update:Partial<Project> = {};

		if(typeof btn.dataSet.id == "number") {
			update.statusId = btn.dataSet.id;
		} else {
			switch(btn.dataSet.id) {
				case 'complete':
					update.completedAt = new DateTime().format("c")
					break;

				case 'in-progress':
					update.completedAt = null;
					update.statusId = null;

					if(!this.project?.startsAt) {
						update.startsAt = (new DateTime()).format("c");
					}

					break;
			}
		}


		jmapds("Project3").update(this.project!.id, update);
	};

	private applyOptionBtnValueToMainBtn(btn:Button) {
		const main = btn.findAncestorByType(Button);
		if(main) {
			main.text = btn.text;
			main.style.backgroundColor = btn.style.backgroundColor;
		}
	}

	buildStatusMenu() {
		this.menu!.items.replace(...Object.values(this.statuses).filter(s => !s.readonly).map(status => {
			return btn({
				dataSet: {
					id: status.id
				},
				text: status.name,
				style: {
					backgroundColor: "#"+status.color
				},
				handler: this.menuHandler
			})
		}))
	}
}