import {
	checkbox, CheckboxField,
	checkboxselectcolumn,
	column,
	Config,
	createComponent,
	DataSourceStore,
	DefaultEntity,
	Filter,
	ListEventMap, Notifier, ObjectUtil,
	store,
	t,
	Table, Window
} from "@intermesh/goui";
import {statuses} from "./Statuses.js";
import {jmapds} from "@intermesh/groupoffice-core";

const staticStatusCount = Object.values(statuses).length;

export class StatusFilter extends Table {
	private book?: DefaultEntity;
	constructor() {

		const columns = [

			checkboxselectcolumn({
				id: "isVisible",
				listeners: {
					render: ( {result, record}) => {
						const cb = result as CheckboxField;
						cb.color = "#" + record.color;
					}
				}
			}),
			column({
				header: t("Name"),
				id:"name",
				resizable: true,
				sortable: true
			})
		];

		super(store({}), columns);

		this.rowSelectionConfig =  {
			multiSelect: true
		};

		this.headers = false;

		this.fitParent = true;

		this.rowSelection!.on("selectionchange", ({target}) => {
			// don't update when store is loading
			if(!this.book || this.store.loading) {
				return;
			}

			const u:any = {};

			let statusHasChanges = false;

			const statusRecords = this.book.statuses.map((s:any, index:number) => {
				//first 3 indexes are upcoming, in progress and completed
				const isVisible = target.isSelected(s);
				if(isVisible != s.isVisible) {
					statusHasChanges = true;
				}
				return {id: s.id, isVisible: isVisible};
			})


			const filter = structuredClone(this.book.filter) ?? {};

			Object.assign(filter,{
				upcoming: target.isSelected(statuses['upcoming']),
				inprogress: target.isSelected(statuses['in-progress']),
				completed: target.isSelected(statuses['complete']),
			});

			if(JSON.stringify(this.book.filter) != JSON.stringify(filter)) {
				u.filter = filter;
			}

			if(statusHasChanges) {
				u.statuses = statusRecords;
			}
			if(Object.keys(u).length) {
				jmapds("ProjectBook").update(this.book.id, u).catch(e => void Window.error(e));
			}
		})
	}

	public getBook() {
		return this.book!
	}

	public setBook(book:DefaultEntity) {

		this.book = book;

		const s = Object.values(statuses).concat(book.statuses);

		this.store.loadData(s, false);

		if(book.filter) {
			if(book.filter.upcoming) {
				this.rowSelection!.add(statuses["upcoming"])
			}

			if(book.filter.inprogress) {
				this.rowSelection!.add(statuses["in-progress"]);
			}

			if(book.filter.completed) {
				this.rowSelection!.add(statuses["complete"]);
			}
		}
		book.statuses.forEach((s:any, index:number) => {
			if(s.isVisible) {
				this.rowSelection!.add(s);
			}
		});


	}


	public applyToProjectStore(store:DataSourceStore) {
		const filter : Filter = {
			operator: "OR",
			conditions: []
		}

		this.rowSelection!.getSelected().forEach(row => {
			const id = row.record.id;

			switch(id){
				case 'complete':
					filter.conditions!.push({complete: true})
					break;

				case 'upcoming':
					filter.conditions!.push({
						operator: "AND",
						conditions: [
							{
								operator: "OR",
								conditions: [{startsAt: "> now"}, {startsAt: null}]
							},
							{complete: false}
						]
					})
					break;

				case 'in-progress':
					filter.conditions!.push({startsAt: "<= now", complete: false, statusId:null})
					break;

				default:
					filter.conditions!.push({statusId: id});
					break;
			}
		})

		store.setFilter("status",filter);
	}
}

export const statusfilter = (config?: Config<StatusFilter>) => createComponent(new StatusFilter(), config);