<?php

namespace go\modules\business\registration;

use go\core;
use go\core\db\Criteria;
use go\core\controller\User as UserCtlr;
use go\core\model\User;
use go\core\orm\Filters;
use go\core\orm\Query;

/**
 * @copyright (c) 2022, Intermesh BV https://www.intermesh.nl
 * @author Joachim van de Haterd <jvdhaterd@intermesh.nl>
 * @license http://www.gnu.org/licenses/agpl-3.0.html AGPLv3
 */
class Module extends core\Module
{
	public function getAuthor(): string
	{
		return "Joachim van de Haterd <jvdhaterd@intermesh.nl>";
	}

	public function getDependencies(): array
	{
		return ['business/business', 'community/tasks', 'business/projects3'];
	}


	public function requiredLicense(): ?string
	{
		return "groupoffice-pro";
	}

	public function getStatus(): string
	{
		return self::STATUS_BETA;
	}

	public function autoInstall(): bool
	{
		return true;
	}


	public function defineListeners()
	{
		core\model\Principal::on(core\orm\Entity::EVENT_FILTER, static::class, 'onUserFilter');
		UserCtlr::on(UserCtlr::EVENT_BEFORE_CAN_UPDATE, static::class, 'onUserCheckUpdatePerms');
	}

	/**
	 * @todo? make part of the standard business module?
	 * @param Filters $filters
	 * @return void
	 */
	public static function onUserFilter(Filters $filters)
	{
		$filters->add('managedBy', function (Criteria $criteria, $value, Query $query) {
			if ($value) {
				$query->join('business_manager', 'bm', 'bm.subjectId = '.$query->getTableAlias().'.id', 'LEFT');
				if (!$query->isJoined('business_employee', 'e')) {
					$query->join('business_employee', 'e', 'e.id = '.$query->getTableAlias().'.id');
				}
				$criteria->andWhere("(`bm`.`managerId` = ".$value . " OR `e`.`id` = ". $value . ")");
			}
		});

	}

	public static function onUserCheckUpdatePerms(User $user): bool
	{
		if($user->isModified(['employee'])) {
			$managers = $user->employee->managers;
			foreach($managers as $mgr) {
				if($mgr->managerId === go()->getUserId()) {
					return true;
				}
			}
		}

		return false;
	}
}
