import {
	btn,
	datefield,
	DateField,
	DatePicker,
	datepicker,
	DateTime,
	fieldset,
	form,
	t,
	tbar,
	Window
} from "@intermesh/goui";
import {RegistrationEntity} from "./RegistrationEntity.js";
import {statuses} from "./MainColumnView.js";
import {client, jmapds} from "@intermesh/groupoffice-core";

export class CopyRegistrationWindow extends Window {

	private datePicker!: DatePicker;

	constructor(data: Partial<RegistrationEntity>) {
		super();
		Object.assign(this, {
				title: t("Copy time entry"),
				modal: true,
				resizable: false,
				maximizable: false
			}
		);

		const currDate  = (new DateTime(data.startDate)).addDays(1);



		this.items.add(
			form({
					flex: 1,
					cls: "vbox",
					handler: () => {
						const newEntry = data;
						delete newEntry.id;
						delete newEntry.projectName;
						delete newEntry.createdAt;
						delete newEntry.createdBy;
						delete newEntry.modifiedAt;
						delete newEntry.modifiedBy;
						delete newEntry.project;
						delete newEntry.uuid;
						delete newEntry.invoiceId;
						let date = this.datePicker.value;
						newEntry.startDate = date.format("Y-m-d");
						newEntry.endDate = newEntry.startDate;
						newEntry.status = (client.user.isAdmin || (client.user.employee && client.user!.employee.subjects.length > 0)) ? statuses.Approved : statuses.Open;

						jmapds("Registration").create(data)
							.then(() => {
								this.close();
							})
							.catch((e) => {
								Window.alert(
									t("Time overlaps, check each start and end time on this day."),
									t("Warning")
								);
							});
					}
				},

				fieldset({},

					this.datePicker = datepicker({
						style: {margin: "auto"},
						value: currDate,
						listeners: {

						}
					})
				),

				tbar({},
					'->',
					btn({
						type: "submit",
						text: t("Copy")
					})
				))
		)
	}
}