import {MainColumnView} from "./MainColumnView.js";
import {EntityID, t, translate} from "@intermesh/goui";
import {client, modules, router} from "@intermesh/groupoffice-core";
import {Main} from "./Main.js";
import {ListView} from "./ListView.js";

export * from "./RegistrationTable.js";
export * from "./RegistrationEntity.js";
export * from "./RegistrationWindow.js";

modules.register({
	package: "business",
	name: "registration",
	entities:[{
		name: "Registration"
	}],
	async init() {
		client.on("authenticated", ( {session}) => {
			if (!session.capabilities["go:business:registration"]) {
				// User has no access to this module
				return;
			}

			translate.load(GO.lang.core.core, "core", "core");
			translate.load(GO.lang.business.registration, "business", "registration");
			translate.load(GO.lang.business.registration, "business", "projects3");
			translate.load(GO.lang.business.registration, "business", "business");
			translate.load(GO.lang.business.registration, "community", "tasks");
			translate.load(GO.lang.business.registration, "business", "leavedays");

			let registration = new Main();
			router.add(/^registration\/?(month|list|week)?\/?([0-9]+)?\/?([0-9]+)?\/?(notbillable|billable|billed|all)?\/?([^\/]+)?/, (viewMode, projectId:EntityID, employeeId:EntityID, invoiceFilter:string, date:string) => {

				// Restore view if coming back after switching modules
				let currentView = registration.items.get(0);
				if(!viewMode) {
					if(currentView instanceof ListView) {
						viewMode = "list";
						if(!employeeId) {
							employeeId = currentView.employeeCombo.value as string;
						}
						if(!projectId) {
							projectId = currentView.projectCombo.value as string;
						}
					} else if(currentView instanceof MainColumnView) {
						viewMode = currentView.viewModeRadioGrp.value as string;
						if(!employeeId) {
							employeeId = currentView.employeeCombo.value as string;
						}
					} else {
						viewMode = "week";
					}
				}

				if(!invoiceFilter) {
					invoiceFilter = "all";
				}

				if(viewMode == "list") {
					let lv ;
					if(!(currentView instanceof ListView)) {
						lv = new ListView();
						registration.items.replace(lv);
					} else
					{
						lv = currentView;
					}

					lv.viewModeRadioGrp.value = viewMode;

					lv.projectCombo.value = projectId;
					lv.employeeCombo.value = employeeId;

					lv.invoiceFilter.value = invoiceFilter;

					if(date) {
						lv.dateRangeField.value = date;
					}


				} else {
					let cv;
					if(!(currentView instanceof MainColumnView)) {
						cv = new MainColumnView();
						registration.items.replace(cv);
					} else {
						cv = currentView;
					}

					if(employeeId === undefined) {
						employeeId = client.user.id;
					}

					cv.viewModeRadioGrp.value = viewMode;
					cv.employeeCombo.value = employeeId;

				}

				modules.openMainPanel("registration");

				router.setPath("registration", viewMode, projectId, employeeId, invoiceFilter, date);
			})

			modules.addMainPanel("business", "registration", "registration", t("Registration"), () => {

				//this will lazy load Registration when module panel is opened.
				return registration;
			});

		});
	}
});