import {
	btn,
	combobox,
	ComboBox,
	comp,
	Component, DateRangeField,
	daterangefield,
	menu,
	radio,
	RadioField,
	searchbtn,
	splitter,
	t,
	tbar
} from "@intermesh/goui";
import {RegistrationTable} from "./RegistrationTable.js";
import {EmployeeCombo, employeecombo} from "@intermesh/business/business";
import {jmapds, modules, router} from "@intermesh/groupoffice-core";
import {ReportWindow} from "./ReportWindow";


export class ListView extends Component {
	private regTable: RegistrationTable;
	private filterPanel: Component;
	public employeeCombo!: EmployeeCombo;
	public readonly viewModeRadioGrp: RadioField;
	public projectCombo!: ComboBox;
	public invoiceFilter!: RadioField;
	public dateRangeField!: DateRangeField;
	constructor() {
		super();

		this.regTable = this.createTable();
		this.filterPanel = this.createFilterPanel();

		this.cls = "fit hbox";

		this.items.add(

			this.filterPanel,
			splitter({
				resizeComponent: this.filterPanel
			}),
			comp({flex: 1, cls: "vbox"},
				tbar({},
					comp({flex: 1}),


					comp({flex:1, style: {display: "flex", alignItems:"center", justifyContent: "center"}},

						this.viewModeRadioGrp = radio({
							type: "button",
							name: "gridView",
							value: "list",
							listeners: {
								change: ({newValue}) => {
									router.goto("registration", newValue ,this.projectCombo.value, this.employeeCombo.value);
								}
							},
							options: [
								{
									text: t("Week"),
									icon: "view_week",
									value: "week"
								},
								{
									text: t("Month"),
									icon: "view_module",
									value: "month"
								},
								{
									icon: "list",
									text: t("List"),
									value: "list"
								}
							]
						}),
					),

						comp({flex: 1, cls: "hbox"},
							comp({flex: 1}),

							searchbtn({
								listeners: {
									input: ( {text}) => {
										this.regTable.store.setFilter("text", {text: text});
										void this.regTable.store.load();
									}
								}
							}),

							btn({
								icon: "more_vert",
								menu: menu({},

									btn({
										text: t("Import"),
										icon: "cloud_upload",
										handler: () => {
											go.util.importFile(
												'Registration',
												".csv, .json, .xlsx",
												{},
												{}
											);
										}
									}),

									btn({
										icon: "cloud_download",
										text: t("Export"),
										menu: menu({},

											btn({
												icon: "csv",
												text: "Comma Separated Values (*.csv)",
												handler: () => {
													go.util.exportToFile(
														'Registration',
														{filter: this.regTable.store.queryParams.filter},
														'csv');
												}
											}),

											btn({
												icon: "unknown_document",
												text: "Microsoft Excel (*.xlsx)",
												handler: () => {
													go.util.exportToFile(
														'Registration',
														{filter: this.regTable.store.queryParams.filter},
														'xlsx');
												}
											})

											)
									}),

									"-",

									btn({
										icon: "description",
										text: t("Report"),
										handler: () => {
												const win = new ReportWindow();
												win.show();
										}
									})


								),


							}),

						)


					),
				comp({cls: "scroll bg-lowest", flex: 1},
					this.regTable
				)
			)

		);

		this.on("render", () => {
			void this.regTable.store.load();
		})
	}

	private createTable() {
		return new RegistrationTable();
	}

	private updateRouterPath() {
		router.setPath("registration/list", this.projectCombo.value,  this.employeeCombo.value, this.invoiceFilter.value, this.dateRangeField.value);
	}

	private createFilterPanel() {
		return comp({
			width: 370,
			cls: "pad flow"
		},
			this.dateRangeField = daterangefield({
				label: t("Date"),
				listeners: {
					setvalue: ({newValue}) => {
						this.regTable.store.setFilter("startDate", {startDate: newValue})

						if(this.rendered) {
							this.updateRouterPath();
							void this.regTable.store.load();
						}
					}
				}
			}),

			this.employeeCombo = employeecombo({
				disabled: !modules.get("business", "registration")!.userRights.mayManage,
				listeners: {
					setvalue: ({newValue}) => {
						this.regTable.store.setFilter("employeeId", {employeeId: newValue})
						if(this.rendered) {
							void this.regTable.store.load();
							this.updateRouterPath();
						}
					}
				},
				buttons: [
					btn({
						icon: "clear",
						type: "button",
						handler: (btn) => {
							this.employeeCombo.value = undefined;
						}
					})
				],
			}),


			this.projectCombo = combobox({
				dataSource: jmapds("Project3"),
				name: "projectId",
				filterName: "text",
				label: t("Project", "business", "projects3"),
				listeners: {
					setvalue: ({newValue}) => {
						this.regTable.store.setFilter("projectId", {projectId: newValue});

						if(this.rendered) {
							void this.regTable.store.load();
							this.updateRouterPath();
						}
					}
				},
				buttons: [
					btn({
						icon: "clear",
						type: "button",
						handler: (btn) => {
							this.projectCombo.value = undefined;
						}
					})
				],
			}),

			this.invoiceFilter = radio({
				type: "list",
				name: "gridView",
				value: "all",
				listeners: {
					setvalue: ( {newValue}) => {
						const store = this.regTable.store;

						let f = undefined;
						switch(newValue){
							case "notbillable":
								f = {billable: false};
								break;
							case "billed":
								f = {billed: true};
								break;

							case "billable":
								f = {billed: false, billable: true}
								break;
						}

						store.setFilter("invoice", f);
						if(this.rendered) {
							void store.load();
							this.updateRouterPath();
						}


					}
				},
				options: [
					{
						text: t("All"),
						icon: "view_week",
						value: "all"
					},
					{
						text: t("Billed"),
						icon: "done",
						value: "billed"
					},
					{
						icon: "monetization_on",
						text: t("Billable"),
						value: "billable"
					},
					{
						icon: "money_off",
						text: t("Not billable"),
						value: "notbillable"
					}
				]
			}),


		);
	}

}