import {btn, column, comp, DateTime, Store, store, t, Table} from "@intermesh/goui";

interface MonthRecord {
	monthnb: number,
	yearnb: number,
	name: string,
	closed: boolean,
	disapproved: boolean,
	startTime: DateTime
}

export class MonthTable extends Table<Store<MonthRecord>> {

	public year?: number;

	public monthnb?: number;

	public userId?: number;

	constructor() {
		const today = new DateTime();
		const columns = [
			column({
				header: t("Name"),
				id: "name",
				resizable: false,
				sortable: false,
				width: 120,

				renderer: (val, record, td, table, rowIndex) => {
					let cls = "";
					if (record.monthnb === today.getMonth() && record.yearnb === today.getYear()) {
						cls = "today";
					}
					return comp({text: val, cls: cls, flex: 1});
				},
			}),

			column({
				header: t("Icon"),
				id: "disapprovedDisabled",
				resizable: false,
				sortable: false,
				width: 32,
				renderer: (val, record, td, table, rowIndex) => {
					let cmp = comp();
					if (record.disapproved === true) {
						cmp.items.add(btn({icon: "close"}))
					} else if (record.closed) {
						cmp.items.add(btn({icon: "done"}))
						// } else {
						//     cmp.items.add(btn({icon: "more", cls: "accent"}))
					}
					return cmp;
				}
			})
		];

		const monthStore = store<MonthRecord>({
			sort: [{property: "monthnb", isAscending: false}]
		});

		super(monthStore, columns);

		this.headers = false;

		this.rowSelectionConfig = {
			multiSelect: false
		};

	}
}
