import {
	boolcolumn,
	column,
	DataSourceStore,
	datasourcestore,
	DateInterval,
	datetimecolumn,
	Format,
	t,
	Table
} from "@intermesh/goui";
import {JmapDataSource, jmapds, modules} from "@intermesh/groupoffice-core";
import {RegistrationEntity} from "./RegistrationEntity.js";
import {RegistrationWindow} from "./RegistrationWindow.js";


export class RegistrationTable extends Table<DataSourceStore<JmapDataSource<RegistrationEntity>>> {
	constructor() {

		const store = datasourcestore({
			dataSource: jmapds<RegistrationEntity>("Registration"),
			queryParams: {
				limit: 20
			},
			sort: [{property: "startDate", isAscending: false}],
			relations: {
				employee: {
					path: "employeeId",
					dataSource: jmapds("Principal")
				},
				project: {
					path: "projectId",
					dataSource: jmapds("Project3")
				},
				activity: {
					path: "activityId",
					dataSource: jmapds("Activity")
				},
				invoice: {
					path: "invoiceId",
					dataSource: jmapds("FinanceDocument")
				}

			}
		});

		const columns = [

			column({
				header: "ID",
				id: "id",
				resizable: true,
				width: 80,
				sortable: true,
				align: "right",
				hidden: true
			}),

			datetimecolumn({
				header: t("Date"),
				id: "startDate",
				sortable: true,
				renderer:(columnValue, record, td) => {
					return Format.dateTime(columnValue + " " + record.startTime);
				}
			}),

			column({
				header: t("Duration"),
				id: "bookedMinutes",
				sortable: true,
				width: 100,
				align: "right",
				renderer:columnValue => {
					const di = DateInterval.createFromFormat(columnValue, "j");
					return di ? di.format("h:I") : "-";
				}
			}),
			column({
				header: t("Employee"),
				id: "employee/name",
				resizable: true,
				width: 200,
				sortable: false
			}),
			column({
				header: t("Project"),
				id: "project",
				renderer: (v) => {
					return v ? v.number + ": " + v.name : "-";
				},
				resizable: true,
				width: 200,
				sortable: false
			}),
			column({
				header: t("Activity"),
				id: "activity/name",
				resizable: true,
				width: 200,
				sortable: false
			}),
			column({
				header: t("Comment"),
				id: "comment",
				resizable: true,
				width: 312,
				sortable: true
			}),
			boolcolumn({
				id:"billed",
				header: t("Billed")
			}),

			boolcolumn({
				id:"billable",
				header: t("Billable")
			})
		];

		if(modules.isAvailable("business", "finance")) {
			columns.push(
				column({
					id:"invoice/number",
					header: t("Invoice"),
					width: 200,
					sortable: false
				})
			);
		}

		super(store, columns);

		this.scrollLoad = true;

		this.rowSelectionConfig = {
			multiSelect: true
		};

		this.on("rowdblclick", ({target, storeIndex}) => {
			const id = target.store.get(storeIndex)!.id;

			const win = new RegistrationWindow();
			void win.load(id);
			win.show();
		})

	}
}