import {
	btn,
	Button, cardmenu, cards, checkbox, checkboxselectcolumn,
	column,
	comp, datasourcestore,
	DateTime,
	Format, h3, RowSelectConfig,
	splitter,
	Store,
	t,
	Table,
	table,
	tbar,
	Window
} from "@intermesh/goui";
import {client, filterpanel, jmapds} from "@intermesh/groupoffice-core";

export class ReportWindow extends Window {
	private readonly monthsTable: Table;
	private readonly weeksTable: Table;
	private params: Record<string, any> = {};
	private yearBtn: Button;
	private projectBookFilter: Table<any>;

	constructor() {
		super();

		this.title = t("Time tracking report");

		this.resizable = true;
		this.maximizable = true;
		this.maximized = true;

		this.width = 1000;
		this.height = 800;

		this.monthsTable = this.createMonthsTable();
		this.weeksTable = this.createWeeksTable();
		this.items.add(
			comp({cls: "hbox", flex: 1},

				comp({width: 300, itemId: "west"},

					tbar({},

						comp({cls: "group", flex: 1},
							btn({icon: "chevron_left", handler: () => {
									this.params.year--;
									this.load();
								}
							}),

							this.yearBtn = btn({text: "2024", flex: 1}),

							btn({
								icon: "chevron_right", handler: () => {
									this.params.year++;
									this.load();
								}
							})
						)
					),

					filterpanel({
						entityName: "Project3",
						listeners: {
							filterchange: ({filter}) => {
								this.params.projectFilter = filter;
								void this.load();
							},

							variablefilterchange: ( {filter}) => {
								this.params.projectVariableFilter = filter;
								void this.load();
							}
						}
					}),

					tbar({cls: "border-bottom"},
						checkbox({
							listeners: {
								change: ( {newValue}) => {

									if (newValue) {
										this.projectBookFilter.rowSelection!.selectAll();
									} else {
										this.projectBookFilter.rowSelection!.clear();
									}

								}
							}
						}),
						h3(t("Project books"))
					),
					this.projectBookFilter = this.createProjectBookFilter()
				),

				splitter({
					resizeComponent: s => s.previousSibling()!
				}),

				comp({cls: "vbox", flex: 1,},
					cardmenu({}),
					cards({
							flex: 1,
							cls: "fit scroll bg-lowest",
							style: {
								scrollPaddingRight: "8rem" // fix for scrollIntoView and sticky column
							},
							listeners: {
								cardchange: () => {
									void this.load();
								}
							}
						},
						this.monthsTable,
						this.weeksTable
					)
				)

			)
		);

		this.on("show", this.onShow, {bind: this});
	}


	private createMonthsTable() {
		return table({
			title: t("Months"),
			fitParent: true,
			store: new Store(),
			columns: [
				column({
					id: "employee",
					header: t("Employee"),
					width: 200,
					sticky: true,
				}),
				column({
					id: "month_1",
					header: DateTime.monthNames[0],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_2",
					header: DateTime.monthNames[1],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_3",
					header: DateTime.monthNames[2],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_4",
					header: DateTime.monthNames[3],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_5",
					header: DateTime.monthNames[4],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_6",
					header: DateTime.monthNames[5],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_7",
					header: DateTime.monthNames[6],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_8",
					header: DateTime.monthNames[7],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_9",
					header: DateTime.monthNames[8],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_10",
					header: DateTime.monthNames[9],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_11",
					header: DateTime.monthNames[10],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					id: "month_12",
					header: DateTime.monthNames[11],
					renderer: Format.duration,
					align: "right",
					width: 80
				}),
				column({
					sticky: true,
					id: "total",
					header: t("Total"),
					renderer: Format.duration,
					align: "right",
					width: 80
				})
			]
		})
	}


	private buildWeekColumns() {
		const columns =  [
			column({
				id: "employee",
				header: t("Employee"),
				width: 200,
				sticky: true,
			})];


		const dt = new DateTime();
		const currentYear = dt.getYear(),
			currentWeek = dt.format("W");
		if(!this.params.year) {
			this.params.year = currentYear
		}

		dt.setYear(this.params.year);
		dt.setDate(1);
		dt.setMonth(1);
// debugger;
		while(dt.getYear() == this.params.year){
			const week = dt.format("W");

			const cls = (currentYear + "-"+ currentWeek) == (this.params.year + "-" + week) ? "primary" : "";

			columns.push(column({
				cls,
				id: "week_" + week,
				header: "w"+ week+ " <small>" + dt.format("d-M") + "</small>",
				align: "right",
				renderer: Format.duration,
				width: 80
			}))
			dt.addDays(7);


			// if( dt.format("W") == "1") {
			// 	break;
			// }
		}

		columns.push(column({
			sticky: true,
			id: "total",
			header: t("Total"),
			renderer: Format.duration,
			align: "right",
			width: 80
		}));

		return columns;
	}

	private createWeeksTable() {

		const columns = this.buildWeekColumns();

		return table({
			title: t("Weeks"),
			fitParent: true,
			store: new Store(),
			columns,
			listeners: {
				render: ({target}) => {
					const w = (new DateTime).format("W");
					target.scrollToColumn("week_" + w);
				},
				show: ({target}) => {
					const w = (new DateTime).format("W");
					target.scrollToColumn("week_" + w);
				}
			}
		})
	}

	private onShow() {

		this.params.year = (new Date).getFullYear();
		void this.load();
	}

	public async load() {

		const months =  !this.monthsTable.hidden;

		const table = months ? this.monthsTable : this.weeksTable;

		const result = await this.mask(client.jmap( months ? "Registration/userMonthReport" : "Registration/userWeekReport", this.params));

		this.yearBtn.text = this.params.year;

		const data: any = {};

		result.registrations.forEach((r: any) => {
			if (!data[r.employeeId]) {
				data[r.employeeId] = {total: 0};
			}

			const mon = months ? "month_" + r.month : "week_" + r.week;
			if (!data[r.employeeId][mon]) {
				data[r.employeeId][mon] = r.bookedMinutes;
			} else {
				data[r.employeeId][mon] += r.bookedMinutes;
			}

			data[r.employeeId]["total"] += r.bookedMinutes;

		})

		const records = [];
		for(const employeeId in data) {
			const r = data[employeeId];
			r.employee = (await jmapds("Principal").single(employeeId))!.name;

			records.push(r);
		}

		console.log(records);

		if(!months) {
			table.columns = this.buildWeekColumns();
		}

		table.store.loadData(records, false);

	}

	private createProjectBookFilter() {
		const store = datasourcestore({
			dataSource: jmapds("ProjectBook"),
			queryParams: {
				limit: 20,
				filter: {
				}
			},
			sort: [{property: "name", isAscending: true}]
		});

		const columns = [
			checkboxselectcolumn({
				id: "selected"
			}),
			column({
				header: t("Name"),
				id:"name",
				resizable: true,
				sortable: true
			})
		];

		return table({
			headers: false,
			cls: "bg-lowest",
			fitParent: true,
			store,
			columns,
			rowSelectionConfig: {
				multiSelect: true,
				listeners: {
					selectionchange: ( {selected}) => {
						this.params.books = selected.map(row => row.id);
						void this.load();
					}
				}
			},
			listeners: {
				render: ({target}) => {
					void target.store.load();
				}
			}
		})
	}
}