import {btn, column, comp, DateTime, Store, store, StoreRecord, t, Table} from "@intermesh/goui";


interface WeekRecord extends StoreRecord {
	weeknb: number,
	yearnb: number,
	name: string,
	closed: boolean,
	disapproved: boolean,
	startTime: DateTime
}


export class WeekTable extends Table<Store<WeekRecord>> {

	public year?: number;

	public weeknb?: number;

	public userId?: number;


	constructor() {
		const today = new DateTime();

		const columns = [
			column({
				header: t("Name"),
				id: "name",
				resizable: false,
				sortable: false,
				width: 120,

				renderer: (val, record, td, table, rowIndex) => {

					return comp({
						text: val,
						cls: record.weeknb === today.getWeekOfYear() && record.yearnb === today.getYear() ? "today" : "",
						flex: 1
					});
				},

			}),

			column({
				header: t("Icon"),
				id: "disapprovedDisabled",
				resizable: false,
				sortable: false,
				width: 32,
				renderer: (val, record, td, table, rowIndex) => {
					let cmp = comp();
					if (record.disapproved === true) {
						cmp.items.add(btn({icon: "close"}))
					} else if (record.closed) {
						cmp.items.add(btn({icon: "done"}))
						// } else {
						//     cmp.items.add(btn({icon: "more", cls: "accent"}))
					}
					return cmp;
				}
			})
		];

		const weekStore = store<WeekRecord>({
			sort: [{property: "weeknb", isAscending: false}]
		});

		super(weekStore, columns);

		this.headers = false;

		this.rowSelectionConfig = {
			multiSelect: false
		};

	}
}