/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Field, FieldConfig, FieldEventMap } from "./Field.js";
import { Component } from "../Component.js";
export type ContainerFieldValue = Record<string, any>;
export interface ContainerField<EventMap extends FieldEventMap = FieldEventMap, ValueType extends ContainerFieldValue = ContainerFieldValue> extends Field<EventMap> {
    set value(v: Partial<ValueType>);
    get value(): ValueType;
}
/**
 * Field that contains fields.
 *
 * The value that it returns is an object with the field names as keys.
 */
export declare class ContainerField<EventMap extends FieldEventMap = FieldEventMap, ValueType extends ContainerFieldValue = ContainerFieldValue> extends Field<EventMap> {
    constructor(tagName?: keyof HTMLElementTagNameMap);
    protected baseCls: string;
    hideLabel: boolean;
    protected fireChangeOnBlur: boolean;
    /**
     * Find all form fields
     *
     * @param nameOrItemId If given only items with matching name or itemId are returned
     */
    findFields(nameOrItemId?: string | undefined): Field<FieldEventMap>[];
    /**
     * Find form field by name or item ID
     *
     * It cascades down the component hierarchy.
     *
     * @param nameOrItemId
     */
    findField<FieldType extends Field = Field>(nameOrItemId: string): FieldType | undefined;
    /**
     * @inheritDoc
     */
    reset(): void;
    /**
     * @inheritDoc
     */
    clear(setValue?: boolean): void;
    /**
     * Copies the current value to the reset value. Typically happens when this component was added to a parent and
     * when the form it belongs too loads.
     */
    trackReset(): void;
    isModified(): boolean;
    /**
     * Add some additional values to the form.
     * setting the value will reset the entire form or container. Patching will leave the other values alone.
     * @param v
     */
    patch(v: Partial<ValueType>): this;
    protected internalSetValue(v: Partial<ValueType>): void;
    protected internalGetValue(): ValueType;
    getOldValue(): any;
    protected validate(): void;
    /**
     * Find the first invalid field
     */
    findFirstInvalid(): Field | undefined;
    protected renderControl(): void;
    clearInvalid(): void;
    /**
     * Not needed on container field as the fields within handle this.
     */
    validateOnBlur: boolean;
    protected applyInvalidMsg(): void;
    /**
     * @inheritDoc
     */
    focus(o?: FocusOptions): void;
}
/**
 * Shorthand function to create {@link ContainerField}
 *
 * @param config
 * @param items
 */
export declare const containerfield: (config?: FieldConfig<ContainerField>, ...items: Component[]) => ContainerField<FieldEventMap, ContainerFieldValue>;
//# sourceMappingURL=ContainerField.d.ts.map