import { FieldConfig } from "./Field.js";
import { DateTime } from "../../util/index.js";
import { InputField } from "./InputField.js";
export interface TimeField {
    get input(): HTMLInputElement;
}
/**
 * TimeField component
 *
 * Time input based on the browser's locale.
 *
 * @property value Outputs time in "H:i" format. eg. 09:30 or 15:30 {@link DateTime.format}
 *
 * @link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/time
 */
export declare class TimeField extends InputField {
    protected baseCls: string;
    private menu;
    constructor();
    fireChangeOnBlur: boolean;
    protected createInput(): HTMLInputElement | HTMLSelectElement | HTMLTextAreaElement;
    private createMenu;
    protected eventTargetIsInFocus(e: FocusEvent): boolean;
    protected internalRemove(): void;
    protected internalRender(): HTMLElement;
    set step(v: number);
    get step(): number;
    /**
     * The minimum number allowed
     *
     * The value of the time input is always in 24-hour format that includes leading zeros: hh:mm
     *
     * @param min
     */
    set min(min: string);
    get min(): string;
    /**
     * The maximum number allowed
     *
     * The value of the time input is always in 24-hour format that includes leading zeros: hh:mm
     *
     * @param max
     */
    set max(max: string);
    get max(): string;
    protected outputFormat(): string;
    /**
     * Get the date as DateTime object
     */
    getValueAsDateTime(): DateTime | undefined;
    set value(v: string | undefined);
    get value(): string | undefined;
}
/**
 * Shorthand function to create {@link TextField}
 *
 * @param config
 */
export declare const timefield: (config?: FieldConfig<TimeField>) => TimeField;
//# sourceMappingURL=TimeField.d.ts.map