import { BaseEntity, Button, CardContainer, Component, DefaultEntity, EntityID, Toolbar, Window, WindowEventMap } from "@intermesh/goui";
import { SharePanel } from "../permissions";
import { CreateLinkButton } from "./CreateLinkButton";
/**
 * @inheritDoc
 */
export interface FormWindowEventMap extends WindowEventMap {
    /**
     * Fires when the window is shown and loaded with data. Also fires if the dialog is for creating new entities and
     * not loaded.
     *
     * @param window
     */
    ready: {
        currentId: EntityID | undefined;
    };
    addlink: {
        entityName: string;
        entityId: EntityID;
    };
}
export declare abstract class FormWindow<EntityType extends BaseEntity = DefaultEntity, EventMap extends FormWindowEventMap = FormWindowEventMap> extends Window<EventMap> {
    entityName: string;
    readonly form: import("@intermesh/goui").DataSourceForm<EntityType>;
    protected readonly cards: CardContainer;
    protected sharePanel?: SharePanel;
    protected bbar: Toolbar;
    /**
     * The first tab
     *
     * @protected
     */
    protected readonly generalTab: Component;
    private cardMenu;
    protected readonly createLinkBtn: CreateLinkButton;
    /**
     * Enable this for linkable entities. It will show a create link button for new items
     */
    hasLinks: boolean;
    protected submitBtn: Button;
    private readonly browseLinksBtn;
    protected closeOnSave: boolean;
    /**
     * Constructor
     *
     * @param entityName Name of the entity for the datasource form
     * @protected
     */
    protected constructor(entityName: string);
    protected onBeforeClose(byUser: boolean): boolean | undefined;
    protected onShow(): void;
    /**
     * Add a share panel to set permissions
     *
     * @params options if not provided the default is:
     * [
     * 	{value: "", name: ""},
     * 	{value: 10, name: t("Read")},
     * 	{value: 20, name: t("Create")},
     * 	{value: 30, name: t("Write")},
     * 	{value: 40, name: t("Delete")},
     * 	{value: 50, name: t("Manage")}
     * ]
     */
    protected addSharePanel(levels?: {
        [key: string]: any;
    }[]): void;
    load(id: EntityID): Promise<this>;
    protected addCustomFields(): void;
    private renderCustomFields;
    /**
     * Adds a link between two entities on save.
     *
     * @param entityName - The name of the target entity.
     * @param  entityId - The ID of the target entity.
     */
    addLinkOnSave(entityName: string, entityId: string): void;
}
//# sourceMappingURL=FormWindow.d.ts.map