/**
 * Recurrenence rule inspired by:
 * https://github.com/kewisch/ical.js/blob/main/lib/ical/recur_iterator.js
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 * Portions Copyright (C) Philipp Kewisch
 */
import { DateTime, Timezone } from "@intermesh/goui";
interface RecurrenceConfig {
    rule: RecurrenceRule;
    timeZone: Timezone;
    dtstart: Date;
}
type NDay = {
    day: string;
    nthOfPeriod?: number;
};
export type Frequency = "yearly" | "monthly" | "weekly" | "daily";
type DayOfWeek = 'mo' | 'tu' | 'we' | 'th' | 'fr' | 'sa' | 'su';
export type RecurrenceRule = {
    frequency: Frequency;
    interval?: number;
    skip?: 'omit' | 'backward' | 'forward';
    firstDayOfWeek?: DayOfWeek;
    count?: number;
    until?: string;
    byDay?: NDay[];
    byMonthDay?: number[];
    byMonth?: string[];
    bySetPosition?: number[];
    byWeekNo?: number[];
    byYearDay?: number[];
    byHour?: number[];
};
/**
 * Class for looping date for Recurrence Rule
 *
 * @category Utility
 */
export declare class Recurrence {
    private rrule?;
    private timeZone?;
    private config;
    private dayNb;
    constructor(config: RecurrenceConfig);
    private makeDate;
    loop(start: DateTime, end: DateTime, iter?: (d: Date, i: number) => boolean): Generator<DateTime, void, unknown>;
}
export {};
//# sourceMappingURL=Recurrence.d.ts.map