import { Component } from "./Component";
export class AbstractMenu extends Component {
    constructor() {
        super("menu");
        /**
         * Used by keyboard nav
         * @protected
         */
        this.orientation = "horizontal";
        this.focusedItemIndex = -1;
        this.setupKeyboardNav();
    }
    /**
     * Find the first menu in the tree of submenu's
     */
    findToolbar() {
        let parent = this.parent;
        if (!parent || !(parent instanceof AbstractMenu)) {
            return undefined;
        }
        if (parent.orientation == "horizontal") {
            return parent;
        }
        else {
            return parent.findToolbar();
        }
    }
    setupKeyboardNav() {
        this.items.on("add", (collection, item, index) => {
            item.el.addEventListener("click", () => {
                this.focusedItemIndex = index;
            });
        });
        this.on("hide", () => {
            this.focusedItemIndex = -1;
        });
        this.el.addEventListener('keydown', (ev) => {
            switch (ev.key) {
                case 'ArrowRight':
                    if (this.orientation == "vertical") {
                        if (!this.focusChild()) {
                            const tb = this.findToolbar();
                            if (tb) {
                                tb.focusNext();
                            }
                        }
                    }
                    else {
                        this.focusNext();
                    }
                    // ev.stopPropagation();
                    ev.preventDefault();
                    break;
                case 'ArrowDown':
                    if (this.orientation == "vertical") {
                        this.focusNext();
                    }
                    else {
                        this.focusChild();
                    }
                    //ev.stopPropagation();
                    ev.preventDefault();
                    break;
                case 'ArrowLeft':
                    if (this.orientation == "vertical") {
                        if (!this.focusParent()) {
                            const tb = this.findToolbar();
                            if (tb) {
                                tb.focusNext(-1);
                            }
                        }
                    }
                    else {
                        this.focusNext(-1);
                    }
                    // ev.stopPropagation();
                    ev.preventDefault();
                    break;
                case 'ArrowUp':
                    if (this.orientation == "vertical") {
                        this.focusNext(-1);
                    }
                    else {
                        this.focusParent();
                    }
                    // ev.stopPropagation();
                    ev.preventDefault();
                    break;
            }
        });
    }
    focusNext(inc = 1) {
        const nextIndex = this.focusedItemIndex + inc;
        this.focusedItemIndex = Math.min(Math.max(nextIndex, 0), this.items.count() - 1);
        if (nextIndex != this.focusedItemIndex) {
            return false;
        }
        const cmp = this.items.get(this.focusedItemIndex);
        if (!cmp.isFocusable()) {
            return this.focusNext(inc);
        }
        else {
            cmp.focus();
            if (this.orientation == 'horizontal') {
                cmp.el.click();
            }
            return true;
        }
    }
    focusChild() {
        const child = this.items.get(this.focusedItemIndex);
        if (!child || !child.menu) {
            return false;
        }
        child.menu.focusNext();
        return true;
    }
    focusParent() {
        const child = this.items.get(this.focusedItemIndex);
        if (!child) {
            return false;
        }
        if (child.parent) {
            child.parent.focus();
        }
        return true;
    }
    focus(o) {
        if (this.focusedItemIndex > -1) {
            this.items.get(this.focusedItemIndex).focus();
        }
        else {
            const i = this.items.get(0);
            if (i) {
                this.focusedItemIndex = 0;
                i.focus(o);
            }
        }
    }
}
//# sourceMappingURL=AbstractMenu.js.map