/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */ import { t } from "../Translate";
import { btn, Button } from "./Button";
import { tbar, tbarItems, Toolbar } from "./Toolbar";
import { assignComponentConfig, comp } from "./Component";
export class OverlayToolbarButton extends Button {
    constructor() {
        super();
        this.handler = (button, ev) => {
            this.mainTbar = button.findAncestorByType(Toolbar);
            if (!this.mainTbar) {
                throw "Search button must be inside a Toolbar";
            }
            this.getOverlayTBar().show();
        };
        this.tbarItemContainer = comp({ flex: 1 });
    }
    close() {
        document.body.removeEventListener("mousedown", this.closeOnClick);
        this.overlayTbar.hide();
        this.mainTbar.show();
        this.focus();
        this.fire("close", this);
    }
    getOverlayTBar() {
        if (!this.overlayTbar) {
            this.closeOnClick = (e) => {
                if (!this.overlayTbar.el.contains(e.target)) {
                    this.close();
                }
            };
            this.overlayTbar = tbar({
                cls: "overlay",
                hidden: true,
                listeners: {
                    show: () => {
                        document.body.addEventListener("mousedown", this.closeOnClick);
                        this.fire("open", this);
                    }
                }
            }, btn({
                icon: "chevron_left",
                title: t("Back"),
                handler: () => {
                    this.close();
                }
            }), this.tbarItemContainer);
            this.overlayTbar.el.addEventListener("keydown", (e) => {
                if (e.key == "Escape") {
                    e.preventDefault();
                    e.stopPropagation();
                    this.close();
                }
            });
            this.mainTbar.items.add(this.overlayTbar);
        }
        return this.overlayTbar;
    }
    get itemContainerEl() {
        return this.tbarItemContainer.el;
    }
}
/**
 * Shorthand function to create {@link OverlayToolbarButton}
 */
export const overlaytoolbarbutton = (config, ...items) => {
    const c = new OverlayToolbarButton();
    if (config) {
        assignComponentConfig(c, config);
    }
    c.items.add(...tbarItems(items));
    return c;
};
//# sourceMappingURL=OverlayToolbarButton.js.map