/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Component } from "./Component.js";
import { Config, Listener, ObservableListenerOpts } from "./Observable.js";
import { OverlayToolbarButton, OverlayToolbarButtonEventMap } from "./OverlayToolbarButton";
/**
 * @inheritDoc
 */
export interface SearchButtonEventMap<Type> extends OverlayToolbarButtonEventMap<Type> {
    input: (searchBtn: Type, text: string) => void;
    reset: (searchBtn: Type) => void;
}
export interface SearchButton extends OverlayToolbarButton {
    on<K extends keyof SearchButtonEventMap<this>, L extends Listener>(eventName: K, listener: Partial<SearchButtonEventMap<this>>[K], options?: ObservableListenerOpts): L;
    un<K extends keyof SearchButtonEventMap<this>>(eventName: K, listener: Partial<SearchButtonEventMap<this>>[K]): boolean;
    fire<K extends keyof SearchButtonEventMap<this>>(eventName: K, ...args: Parameters<SearchButtonEventMap<Component>[K]>): boolean;
}
export declare class SearchButton extends OverlayToolbarButton {
    private searchField;
    private buffer;
    constructor();
    reset(): void;
    private onInput;
}
/**
 * Shorthand function to create {@link SearchButton}
 *
 * @example
 * ```
 * searchbtn({
 * 	listeners: {
 * 		input:(searchBtn, text) => {
 *
 * 			const filtered = records.filter((r) => {
 * 				return !text || r.description.toLowerCase().indexOf(text.toLowerCase()) === 0;
 * 			});
 *
 * 			//simple local filter on the store
 * 			table.store.loadData(filtered, false)
 * 		}
 * 	}
 * })
 * ```
 *
 * @param config
 * @param items
 */
export declare const searchbtn: (config?: Config<SearchButton, SearchButtonEventMap<SearchButton>>) => SearchButton;
//# sourceMappingURL=SearchButton.d.ts.map