/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { btn } from "./Button.js";
import { t } from "../Translate.js";
import { textfield } from "./form/TextField.js";
import { createComponent } from "./Component.js";
import { FunctionUtil } from "../util/FunctionUtil.js";
import { OverlayToolbarButton } from "./OverlayToolbarButton";
export class SearchButton extends OverlayToolbarButton {
    constructor() {
        super();
        this.buffer = 300;
        this.icon = "search";
        this.title = t("Search");
        this.searchField = textfield({
            label: t("Search"),
            flex: 1,
            buttons: [
                btn({
                    type: "button",
                    icon: "clear",
                    handler: () => {
                        this.reset();
                    }
                })
            ]
        });
        this.searchField.on("render", () => {
            this.searchField.input.addEventListener('input', FunctionUtil.buffer(this.buffer, this.onInput.bind(this)));
            this.searchField.el.addEventListener('keydown', (e) => {
                if (e.key == "Enter") {
                    e.preventDefault();
                    e.stopPropagation();
                }
                // if(e.key == "Escape") {
                // 	e.preventDefault();
                // 	e.stopPropagation();
                // 	this.close();
                // }
            });
        });
        this.items.add(this.searchField);
        this.on("open", () => {
            this.searchField.select();
        });
    }
    reset() {
        this.searchField.reset();
        this.close();
        this.fire("reset", this);
        this.fire("input", this, "");
        this.el.classList.remove("accent");
        this.el.classList.remove("filled");
    }
    onInput() {
        this.el.classList.toggle("accent", !!this.searchField.value);
        this.el.classList.toggle("filled", !!this.searchField.value);
        this.fire("input", this, this.searchField.value);
    }
}
/**
 * Shorthand function to create {@link SearchButton}
 *
 * @example
 * ```
 * searchbtn({
 * 	listeners: {
 * 		input:(searchBtn, text) => {
 *
 * 			const filtered = records.filter((r) => {
 * 				return !text || r.description.toLowerCase().indexOf(text.toLowerCase()) === 0;
 * 			});
 *
 * 			//simple local filter on the store
 * 			table.store.loadData(filtered, false)
 * 		}
 * 	}
 * })
 * ```
 *
 * @param config
 * @param items
 */
export const searchbtn = (config) => createComponent(new SearchButton(), config);
//# sourceMappingURL=SearchButton.js.map