import { ChipsField } from "./ChipsField.js";
import { List, listStoreType } from "../List.js";
import { Menu } from "../menu/index.js";
import { Button } from "../Button.js";
import { FieldConfig, FieldEventMap } from "./Field.js";
import { Listener, ObservableListenerOpts } from "../Observable.js";
import { Component } from "../Component.js";
import { storeRecordType } from "../../data/index.js";
export interface AutocompleteChipsEventMap<Type> extends FieldEventMap<Type> {
    /**
     * Fires when suggestions need to load
     *
     * @param form
     */
    autocomplete: (field: Type, input: string) => any;
}
export interface AutocompleteChips<T extends List> extends ChipsField {
    on<K extends keyof AutocompleteChipsEventMap<this>, L extends Listener>(eventName: K, listener: Partial<AutocompleteChipsEventMap<this>>[K], options?: ObservableListenerOpts): L;
    un<K extends keyof AutocompleteChipsEventMap<this>>(eventName: K, listener: Partial<AutocompleteChipsEventMap<this>>[K]): boolean;
    fire<K extends keyof AutocompleteChipsEventMap<this>>(eventName: K, ...args: Parameters<AutocompleteChipsEventMap<Component>[K]>): boolean;
}
/**
 * Chips component that auto completes user input
 */
export declare class AutocompleteChips<T extends List = List> extends ChipsField {
    readonly list: T;
    private buffer;
    protected readonly menu: Menu;
    protected readonly menuButton: Button;
    private valuesToCompare?;
    textInputToValue: (text: string) => Promise<any>;
    /**
     * Constructor
     *
     * @param list The drop down list or {@link Table}.
     * @param buffer
     */
    constructor(list: T, buffer?: number);
    private initList;
    private isPickerRecordInValue;
    /**
     * Method that transforms a record from the TablePicker store to a value for this field.
     * This is not necessarily a text value. In conjunction with {@link valueToTextField()} this
     * could also be an ID of an object for example.
     *
     * @param field
     * @param record
     */
    pickerRecordToValue(field: this, record: storeRecordType<listStoreType<T>>): any;
    protected internalRender(): HTMLElement;
    protected onEditorKeyDown(ev: KeyboardEvent): void;
    protected onEnter(ev: KeyboardEvent): void;
    private addSelected;
    private onInput;
}
type AutoCompleteChipsConfig<ListType extends List = List> = FieldConfig<AutocompleteChips<ListType>, AutocompleteChipsEventMap<AutocompleteChips<ListType>>, "list"> & Partial<Pick<AutocompleteChips<ListType>, "textInputToValue" | "chipRenderer" | "pickerRecordToValue">>;
/**
 * Shorthand function to create {@link AutocompleteChips}
 *
 * @param config
 */
export declare const autocompletechips: <ListType extends List = List<import("../../index.js").Store<import("../../index.js").StoreRecord>>>(config: AutoCompleteChipsConfig<ListType>) => AutocompleteChips<ListType>;
export {};
//# sourceMappingURL=AutocompleteChips.d.ts.map