import { Field } from "./Field.js";
import { comp, Component, createComponent, span } from "../Component.js";
import { btn } from "../Button.js";
import { t } from "../../Translate.js";
class Chip extends Component {
    constructor() {
        super();
        this.cls = "chip";
        this.items.add(this.textComponent = span({}), btn({
            title: t("Remove"),
            icon: "cancel",
            cls: "small",
            handler: (btn) => {
                this.fire("deleteclick", this);
            }
        }));
    }
    get text() {
        return this.textComponent.text;
    }
    set text(text) {
        this.textComponent.text = text;
        this.title = text;
    }
}
/**
 * Chips component
 */
export class ChipsField extends Field {
    constructor() {
        super(...arguments);
        this.baseCls = 'goui-form-field chips';
        /**
         * Function that transforms the user text input to a chip.
         *
         * If it returns an empty value no chip will be rendered
         *
         * @param text
         */
        this.textInputToValue = async (text) => {
            return text;
        };
        /**
         * Renders a value to the chip component
         * @param value
         */
        this.chipRenderer = (chip, value) => {
            chip.text = value;
        };
        this.selectedIndex = -1;
    }
    get editor() {
        return this._editor;
    }
    createControl() {
        this.chipsContainer = document.createElement("div");
        this.chipsContainer.classList.add("control-wrap");
        this._editor = comp({
            id: Component.uniqueID(),
            attr: {
                contentEditable: "true"
            },
            cls: "editor"
        });
        this.el.setAttribute("for", this._editor.id);
        this._editor.el.addEventListener("keydown", (ev) => {
            this.onEditorKeyDown(ev);
        });
        this.el.addEventListener("click", (ev) => {
            if (this.selectedIndex == -1) {
                this._editor.focus();
            }
        });
        this._editor.el.addEventListener("focus", (ev) => {
            this.clearSelection();
        });
        this._editor.el.addEventListener("blur", () => {
            this._editor.el.innerText = "";
        });
        //this._editor.render(this.chipsContainer!);
        this.items.add(this._editor);
        return this.chipsContainer;
    }
    // get value(): FieldValue[] {
    // 	return super.value as FieldValue[];
    // }
    //
    // set value(v: FieldValue[]) {
    // 	super.value = v;
    // }
    onEditorKeyDown(ev) {
        this.clearInvalid();
        switch (ev.key) {
            case "Enter":
                this.onEnter(ev);
                break;
            case "ArrowRight":
                if (this.selectedIndex > -1) {
                    this.select(this.selectedIndex + 1);
                }
                break;
            case "ArrowLeft":
                if (this.selectedIndex == -1) {
                    if (this._editor.html == "") {
                        ev.preventDefault();
                        this.select(this.items.count() - 2);
                    }
                }
                else {
                    this.select(this.selectedIndex - 1);
                }
                break;
            case "Delete":
            case "Backspace":
                ev.stopPropagation();
                if (this._editor.html == "") {
                    if (this.selectedIndex == -1) {
                        ev.preventDefault();
                        this.select(this.items.count() - 2);
                    }
                    else {
                        this.items.get(this.selectedIndex).remove();
                        this.select(this.selectedIndex);
                    }
                    // this.focus();
                }
                break;
        }
    }
    onEnter(ev) {
        ev.preventDefault();
        const chip = this.createChip();
        this.textInputToValue(this._editor.text).then((value) => {
            if (!value) {
                return;
            }
            let r = this.chipRenderer(chip, value);
            if (r instanceof Promise) {
                r.then(() => {
                    this.items.insert(-1, chip);
                    this.value = this.value.concat([value]);
                });
            }
            else {
                this.items.insert(-1, chip);
                this.value = this.value.concat([value]);
            }
        });
        this._editor.text = "";
    }
    createChip() {
        const chip = new Chip();
        chip.on("deleteclick", comp1 => {
            this.captureValueForChange();
            const index = this.items.indexOf(chip);
            chip.remove();
            this.value.splice(index, 1);
            this.fireChange();
        });
        chip.el.addEventListener("click", () => {
            this.select(this.items.indexOf(chip));
            // this.focus();
        });
        return chip;
    }
    get itemContainerEl() {
        return this.chipsContainer;
    }
    clearSelection() {
        if (this.selectedIndex > -1) {
            const item = this.items.get(this.selectedIndex);
            if (item) {
                item.el.classList.remove("selected");
            }
            this.selectedIndex = -1;
        }
    }
    select(index) {
        if (index < 0) {
            return;
        }
        this.clearSelection();
        if (index > (this.items.count() - 2)) {
            this._editor.focus();
            return;
        }
        this.selectedIndex = index;
        this.items.get(this.selectedIndex).el.classList.add("selected");
    }
    internalSetValue(v) {
        //remove all chips except the editor (last item).
        while (this.items.count() > 1) {
            this.items.removeAt(0);
        }
        v.forEach((v) => {
            const chip = this.createChip();
            this.chipRenderer(chip, v);
            chip.dataSet.value = v;
            this.items.insert(-1, chip);
        });
    }
    internalGetValue() {
        const v = [];
        this.items.forEach((item) => {
            if (item.dataSet.value) {
                v.push(item.dataSet.value);
            }
        });
        return v;
    }
    // protected internalRender(): HTMLElement {
    // 	const el = super.internalRender();
    // 	this.renderValue();
    // 	return el;
    // }
    //
    // private renderValue() {
    // 	const v = this.internalGetValue() as Array<any>;
    //
    // 	if(!v) {
    // 		return;
    // 	}
    //
    // 	v.forEach((v:any) => {
    // 		const chip =  this.createChip();
    //
    // 		this.chipRenderer(chip, v).then(() => {
    // 			this.items.insert(-1, chip);
    // 		});
    // 	});
    // }
    focus(o) {
        this.editor.focus(o);
    }
}
/**
 * Shorthand function to create {@link ChipsField}
 *
 * @param config
 */
export const chips = (config) => createComponent(new ChipsField(), config);
//# sourceMappingURL=ChipsField.js.map