/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Michael de Hart <mdhart@intermesh.nl>
 */
import { Field } from "./Field.js";
import { createComponent } from "../Component.js";
export class MapField extends Field {
    constructor(buildField) {
        super('div');
        this.buildField = buildField;
        this.baseCls = "";
        this._nextKey = 0;
        this.cls = "vbox gap";
    }
    renderControl() {
        // empty
    }
    internalSetValue(v) {
        this.items.clear();
        if (v) {
            for (const key in v) {
                this.internalAdd(-1, v[key], key);
            }
        }
    }
    internalGetValue() {
        const v = {};
        this.items.forEach((field) => {
            if (field instanceof Field) {
                const rowValue = field.value;
                let key = field.dataSet.key;
                if (this.keyFieldName && rowValue[this.keyFieldName]) {
                    key = rowValue[this.keyFieldName];
                    delete rowValue[this.keyFieldName];
                }
                v[key] = this.valueFieldName ? rowValue[this.valueFieldName] : rowValue;
            }
        });
        return v;
    }
    isEmpty() {
        return this.items.count() === 0;
    }
    /**
     * Add value to the map.
     *
     * Also fires change event
     *
     * @param data
     * @param key
     */
    add(data, key) {
        this.internalAdd(-1, data, key);
        this.fireChange();
    }
    insert(index, data, key) {
        this.internalAdd(index, data, key);
        this.fireChange();
    }
    internalAdd(index, data, key) {
        if (!this.valueOnFocus) {
            this.captureValueForChange();
        }
        if (typeof key === 'number') {
            this._nextKey = Math.max(this._nextKey, key);
        }
        const field = this.buildField(data);
        field.dataSet.key = key || this.nextKey();
        field.value = this.valueFieldName ? { [this.valueFieldName]: data, [this.keyFieldName]: key } : data;
        index < 0 ? this.items.add(field) : this.items.insert(index, field);
    }
    nextKey() {
        // only works if sorted by key
        // todo: maybe use GUID for item with sortOrder?
        return ++this._nextKey;
    }
}
/**
 * Create a map field
 *
 * @link https://goui.io/#form/MapField
 * @param config
 * @param items
 */
export const mapfield = (config, ...items) => createComponent(new MapField(config.buildField), config, items);
//# sourceMappingURL=MapField.js.map