/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2024 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Component, ComponentEventMap } from "../Component.js";
import { DateTime } from "../../util";
import { Config, Listener, ObservableListenerOpts } from "../Observable.js";
export interface MonthPickerEventMap<Type> extends ComponentEventMap<Type> {
    /**
     * Fires when a Month is selected
     *
     * @param datepicker
     * @param date
     */
    select: (datepicker: Type, date: DateTime) => false | void;
}
export interface MonthPicker extends Component {
    on<K extends keyof MonthPickerEventMap<this>, L extends Listener>(eventName: K, listener: Partial<MonthPickerEventMap<this>>[K], options?: ObservableListenerOpts): L;
    un<K extends keyof MonthPickerEventMap<this>>(eventName: K, listener: Partial<MonthPickerEventMap<this>>[K]): boolean;
    fire<K extends keyof MonthPickerEventMap<this>>(eventName: K, ...args: Parameters<MonthPickerEventMap<Component>[K]>): boolean;
}
/**
 * Month picker
 *
 * Select a Month of the year
 */
export declare class MonthPicker extends Component {
    private _value;
    private now;
    protected baseCls: string;
    private yearField;
    private monthMenu;
    constructor();
    set value(v: DateTime);
    get value(): DateTime;
    private changeYear;
}
/**
 * Create a Month picker
 *
 * Can select a Month of the year
 *
 * @param config
 */
export declare const monthpicker: (config?: Config<MonthPicker, MonthPickerEventMap<MonthPicker>>) => MonthPicker;
//# sourceMappingURL=MonthPicker.d.ts.map