/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2024 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Component, ComponentEventMap } from "../Component.js";
import { DateTime } from "../../util";
import { Config, Listener, ObservableListenerOpts } from "../Observable.js";
export interface WeekPickerEventMap<Type> extends ComponentEventMap<Type> {
    /**
     * Fires when a week is selected
     *
     * @param datepicker
     * @param date
     */
    select: (datepicker: Type, date: DateTime) => false | void;
}
export interface WeekPicker extends Component {
    on<K extends keyof WeekPickerEventMap<this>, L extends Listener>(eventName: K, listener: Partial<WeekPickerEventMap<this>>[K], options?: ObservableListenerOpts): L;
    un<K extends keyof WeekPickerEventMap<this>>(eventName: K, listener: Partial<WeekPickerEventMap<this>>[K]): boolean;
    fire<K extends keyof WeekPickerEventMap<this>>(eventName: K, ...args: Parameters<WeekPickerEventMap<Component>[K]>): boolean;
}
/**
 * Week picker
 *
 * Select a week of the year
 */
export declare class WeekPicker extends Component {
    private _value;
    private now;
    protected baseCls: string;
    private yearField;
    private weekMenu;
    constructor();
    set value(v: DateTime);
    get value(): DateTime;
    private changeYear;
}
/**
 * Create a week picker
 *
 * Can select a week of the year
 *
 * @param config
 */
export declare const weekpicker: (config?: Config<WeekPicker, WeekPickerEventMap<WeekPicker>>) => WeekPicker;
//# sourceMappingURL=WeekPicker.d.ts.map