/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Component, ComponentEventMap } from "../Component.js";
import { Toolbar } from "../Toolbar.js";
import { Table } from "./Table.js";
import { Config } from "../Observable.js";
export declare class MultiSelectToolbar extends Toolbar {
    readonly table: Table;
    private label;
    private backBtn;
    constructor(table: Table);
}
/**
 * Create a {@link MultiSelectToolbar} component
 *
 * You can add this to a {@link Toolbar}
 *
 * @example
 * ```
 * tbar({},
 *    mstbar({
 * 			table: usrTbl.table
 * 		},
 * 	    '->',
 * 			btn({
 * 				icon: "delete",
 * 				handler: async (btn) => {
 * 					const msTB = btn.parent as MultiSelectToolbar, ids = msTB.table.rowSelection!.getSelected().map((row) => row.id);
 * 					const result = await userDS.confirmDestroy(ids);
 * 					if (result != false) {
 * 						msTB.hide();
 * 					}
 * 				}
 * 			})
 * 		)
 * );
 *
 *
 * @param config
 * @param items
 */
export declare const mstbar: (config: Config<MultiSelectToolbar, ComponentEventMap<MultiSelectToolbar>, "table">, ...items: (Component | "->" | "-")[]) => MultiSelectToolbar;
//# sourceMappingURL=MultiSelectToolbar.d.ts.map