import { TableColumn } from "../table";
import { a, comp, createComponent, span } from "../Component";
import { checkbox } from "../form";
export class TreeColumn extends TableColumn {
    constructor() {
        super(...arguments);
        this.renderer = (columnValue, record, td, tree, storeIndex, column) => {
            const node = comp({ cls: "node" });
            if (record.level) {
                for (let i = 0; i < record.level; i++) {
                    node.items.add(comp({ cls: "indent" }));
                }
            }
            node.items.add(span({ cls: "caret" }));
            if (record.icon || this.defaultIcon) {
                node.items.add(comp({ cls: "icon", text: record.icon ?? this.defaultIcon, tagName: "i" }));
            }
            if (record.check !== undefined) {
                const c = checkbox({
                    value: record.check,
                    listeners: {
                        change: (field, newValue, oldValue) => {
                            record.check = newValue;
                            tree.fire("checkchange", this, record, storeIndex, newValue);
                        }
                    }
                });
                node.items.add(c);
            }
            node.items.add(record.href ? a({ href: record.href, text: columnValue, cls: "label" }) : comp({ cls: "label", text: columnValue }));
            if (record.expanded) {
                node.el.classList.add("expanded");
            }
            if (record.children && record.children.length == 0) {
                node.el.classList.add("no-children");
            }
            if (record.cls) {
                node.el.classList.add(record.cls);
            }
            return node;
        };
    }
}
export const treecolumn = (config) => createComponent(new TreeColumn(config.id), config);
//# sourceMappingURL=TreeColumn.js.map